#include "common.h"
#include "dbg.h"
#include <QtGui>
#include <cmath>

#define TO_INT_PRECISION 0.00001

void create_slider_spinbox_pair(QSlider** ppSlider, QDoubleSpinBox** ppSpinbox)
{
	*ppSlider = new QSlider(Qt::Horizontal);
	*ppSpinbox = new QDoubleSpinBox;
}

void set_slider_spinbox_pair(QSlider* pSlider, QDoubleSpinBox* pSpinbox, int min, int max, int precision, double defaultValue, double * value)
{
	*value = defaultValue;
	int values = (max - min) * (int)pow((float)10,precision);
	pSlider->setRange(min * (int)pow((float)10,precision), max * (int)pow((float)10,precision));
	pSlider->setSingleStep((int)(values/50));
	pSlider->setPageStep((int)(values/10));
	pSlider->setTickInterval((int)(values/10));
	pSlider->setTickPosition(QSlider::TicksBelow);
	pSlider->setValue((int)(defaultValue*pow((float)10,precision)));

	pSpinbox->setRange(min, max);
	pSpinbox->setDecimals(2);
	pSpinbox->setValue(defaultValue);
	pSpinbox->setSingleStep(1/pow((float)10,precision));
}

int cmp_float_precision(double a, double b, int precision)
{
	double dif = a - b;
	double thr = (double)1/pow((float)10, precision+1);
//	DBG("MainWindow::CmpFloatPrecision("<<a<<", "<<b<<", "<<precision<<"): dif:"<<dif<<" thr:"<<thr);
	if (dif > 0 && dif > thr)
	{
		return 1;
	} 
	else if (dif < 0 && dif < -thr)
	{
		return -1;
	}
	else
	{
		return 0;
	}
}

int to_int(double f)
{
	int i = (int)f;
	double diff = (double)i - f;
	DBG("to_int("<<f<<") i:"<<i<<" diff:"<<diff);
	if (diff > TO_INT_PRECISION)
	{
		return i-1;
	} 
	else if (diff < -TO_INT_PRECISION)
	{
		return i+1;
	}
	else
	{
		return i;
	}
}

