#ifndef COMPUTESCORETHREAD_H
#define COMPUTESCORETHREAD_H

#include <QMutex>
#include <QSize>
#include <QThread>
#include <QWaitCondition>
#include "mlfmodeldata.h"
#include "dbg.h"
#include "histogramplot.h"
#include "ithreadstatecallback.h"
#include "mlfsortingthread.h"
#include "mlfmodeldata.h"

class ComputeScoreThread : public QThread, public IThreadStateCallback
{
		Q_OBJECT

	public:
	    ComputeScoreThread(QObject *parent = 0);
	    ~ComputeScoreThread();

		void Compute(MlfModelData::EInputScoreType inputScoreType, MlfModelData::EScoreType scoreType, double prior, double multiplicative, double additive, double threshold);
		void SetData(MlfModelData::Data* pData) { mpData = pData; }
		void SetSelectedKeywords(MlfSortingThread::SelectedKeywordsMap* pMap) { mpSelectedKeywords = pMap; }

		HistogramPlot* GetHistogramPlot() { return mpHistogramPlot; }
		void SetHistogramPlot(HistogramPlot* plot) { mpHistogramPlot = plot; }

		bool GetStateRestart() const { return mRestart; }
		bool GetStateAbort() const { return mAbort; }

		static double Llr2Posterior(double llr, double prior = 0);
		static double LogLik2Posterior(double logLik, double prior = 0);
		static double Posterior2Llr(double posterior, double prior = 0);

		static double logit(double p);
		static double sigm(double d);

	signals:
		void ScoreComputed(double min, double max);

	protected:
		void run();

		QMutex mMutex;
		QWaitCondition mCondition;

		bool mRestart;
		bool mAbort;

		MlfModelData::Data * mpData;
		MlfSortingThread::SelectedKeywordsMap * mpSelectedKeywords;

		HistogramPlot * mpHistogramPlot;
		double					mNormalizedScoreMin;
		double					mNormalizedScoreMax;
		float 					mHistogramItemWidth;
		QVector<unsigned int>	mHistogramScores;
		unsigned int			mHistogramMaxCount;

		MlfModelData::EInputScoreType  mInputScoreType;
		MlfModelData::EScoreType 	   mScoreType;
		double 					mPrior;
		double 					mMultiplicative;
		double 					mAdditive;
		double					mThreshold;
};

#endif

