#include "histogramplot.h"
#include <QtGui>

#define CALL_MEMBER_FN(object,ptrToMember)  ((object).*(ptrToMember)) 

HistogramPlot::HistogramPlot(QWidget* parent) : QFrame(parent), mThreshold(0), mFixedScoreInterval(false)
{
//	mHistogramType = histogramType;
	mHistogramType = EHistogramTypeIgiAccumulative;
	for (int i=0; i<EHistogramTypeIndexCOUNT; i++)
	{
		mpPathHits[i] = NULL;
		mpPathMisses[i] = NULL; 
		mpPathFalseAlarms[i] = NULL;
	}
	setMinimumWidth(100);
	setMinimumHeight(100);
	setMouseTracking(true);
	mCursorShow = false;
	mReferenceMisses = 0;
	mZoom = 1.0;
	DBG("mReferenceMisses:"<<mReferenceMisses<<" this:"<<(int)this);
}

void HistogramPlot::Reset(double xMin, double xMax)
{
	DBG("mReferenceMisses:"<<mReferenceMisses<<" this:"<<(int)this);
	mReady = false;
	if (!mFixedScoreInterval)
	{
		mXMin[EHistogramTypeIndexIgiAccumulative] = xMin;
		mXMax[EHistogramTypeIndexIgiAccumulative] = xMax;
//		DBG("mXMax:"<<mXMax<<" mXMin:"<<mXMin<<" mHistogramItemsCount:"<<mHistogramItemsCount);
	}
	mXMin[EHistogramTypeIndexLukasAccumulative] = -15;
	mXMax[EHistogramTypeIndexLukasAccumulative] = 15;
	mXMin[EHistogramTypeIndexLukas] = -15;
	mXMax[EHistogramTypeIndexLukas] = 15;


	for (int i=0; i<EHistogramTypeIndexCOUNT; i++)
	{
		mHistogramItemWidth[i] = (mXMax[i] - mXMin[i]) / (mHistogramItemsCount-1);
	}
	DBG("mHistogramItemWidth:"<<mHistogramItemWidth);
	DBG("mReferenceMisses:"<<mReferenceMisses<<" this:"<<(int)this);
//	mHistogramScores.resize(mHistogramItemsCount);
	mHistogramHits.resize(mHistogramItemsCount);
	mHistogramMisses.resize(mHistogramItemsCount);
	mHistogramFalseAlarms.resize(mHistogramItemsCount);
	for (int j=0; j<EHistogramTypeIndexCOUNT; j++)
	{
		mHistogramMaxCount[j] = mReferenceMisses;
	}
	for (int i=0; i<mHistogramHits.size(); i++)
	{
//		mHistogramScores[i] = 0;
		mHistogramHits[i].resize(EHistogramTypeIndexCOUNT);
		mHistogramMisses[i].resize(EHistogramTypeIndexCOUNT);
		mHistogramFalseAlarms[i].resize(EHistogramTypeIndexCOUNT);
		for (int j=0; j<EHistogramTypeIndexCOUNT; j++)
		{
			mHistogramHits[i][j] = 0;
			mHistogramMisses[i][j] = mReferenceMisses;
			mHistogramFalseAlarms[i][j] = 0;
		}
	}
}

HistogramPlot::EDetectionType HistogramPlot::GetRowType(MlfModelData::ECorrect correct, double score, EHistogramTypeIndex typeIndex, int histogramIndex) const
{
	float thr = IndexToScore(typeIndex, histogramIndex);
	float thr_low = thr - mHistogramItemWidth[typeIndex]/2;
	float thr_high = thr + mHistogramItemWidth[typeIndex]/2;
	EDetectionType type = HistogramPlot::DetectionUnknown;
	switch (typeIndex)
	{
		case EHistogramTypeIndexIgiAccumulative: 
		case EHistogramTypeIndexLukasAccumulative:
			if (correct == MlfModelData::CorrectYes && score >= thr)
			{
				type = HistogramPlot::DetectionHit;
			}
			else if (correct == MlfModelData::CorrectNo && score >= thr)
			{
				type = HistogramPlot::DetectionFalseAlarm;
			}
			else if (correct == MlfModelData::CorrectYes && score < thr)
			{
				type = HistogramPlot::DetectionMiss;
			}
			break;

		case EHistogramTypeIndexLukas:
			if (correct == MlfModelData::CorrectYes && score >= thr_low && score < thr_high)
			{
				type = HistogramPlot::DetectionHit;
			}
			else if (correct == MlfModelData::CorrectNo && score >= thr_low && score < thr_high)
			{
				type = HistogramPlot::DetectionFalseAlarm;
			}
			break;
		case EHistogramTypeIndexCOUNT:
			break;
	}
	return type;
}

void HistogramPlot::AddData(MlfModelData::MlfTableRow * pRow)
{
	mReady = false;
//	DBG("score:" << score << " index:"<<GetHistogramIndex(score) << " mXMin:"<<mXMin<<" mHistogramItemWidth:"<<mHistogramItemWidth);
	
	// EHistogramTypeIgiAccumulative
	{
		double score = pRow->mpMlfRecord->mScore2;
		if (GetHistogramIndex(EHistogramTypeIndexIgiAccumulative, score) < 0 || GetHistogramIndex(EHistogramTypeIndexIgiAccumulative, score) >= mHistogramHits.size())
		{
			return;
		}
		unsigned int count = 0;
		for (int i=0; i<mHistogramHits.size(); i++)
		{
			EDetectionType type = GetRowType(pRow->mCorrect, score, EHistogramTypeIndexIgiAccumulative, i);
			switch(type)
			{
				case DetectionHit: 
					count = ++mHistogramHits[i][EHistogramTypeIndexIgiAccumulative]; 
					break;

				case DetectionMiss: 
					count = ++mHistogramMisses[i][EHistogramTypeIndexIgiAccumulative]; 
					if (mHistogramMaxCount[EHistogramTypeIndexIgiAccumulative] < count)
					{
						mHistogramMaxCount[EHistogramTypeIndexIgiAccumulative] = count;
					}
					break;

				case DetectionFalseAlarm: 
					count = ++mHistogramFalseAlarms[i][EHistogramTypeIndexIgiAccumulative]; 
					break;

				case DetectionUnknown: 
					break;
			}
		}
	}
	// EHistogramTypeLukasAccumulative
	{
		double score = pRow->mScoreLlrNorm;
		// maximum number of misses in case the threshold is infinite
		if (pRow->mCorrect == MlfModelData::CorrectYes)
		{
			++mHistogramMaxCount[EHistogramTypeIndexLukasAccumulative];
		}
		if (GetHistogramIndex(EHistogramTypeIndexLukasAccumulative, score) < 0 || GetHistogramIndex(EHistogramTypeIndexLukasAccumulative, score) >= mHistogramHits.size())
		{
			return;
		}
		unsigned int count = 0;
		for (int i=0; i<mHistogramHits.size(); i++)
		{
			EDetectionType type = GetRowType(pRow->mCorrect, score, EHistogramTypeIndexLukasAccumulative, i);
			switch(type)
			{
				case DetectionHit: 
					count = ++mHistogramHits[i][EHistogramTypeIndexLukasAccumulative]; 
					break;

				case DetectionMiss: 
					count = ++mHistogramMisses[i][EHistogramTypeIndexLukasAccumulative]; 
					break;

				case DetectionFalseAlarm: 
					count = ++mHistogramFalseAlarms[i][EHistogramTypeIndexLukasAccumulative]; 
					break;

				case DetectionUnknown: 
					break;
			}
		}
	}
	// EHistogramTypeLukas
	{
		double score = pRow->mScoreLlrNorm;
		int i = GetHistogramIndex(EHistogramTypeIndexLukas, score);
		if (i < 0 || i >= mHistogramHits.size())
		{
			return;
		}
		unsigned int count = 0;
		EDetectionType type = GetRowType(pRow->mCorrect, score, EHistogramTypeIndexLukas, i);
		switch(type)
		{
			case DetectionHit: 
				count = ++mHistogramHits[i][EHistogramTypeIndexLukas]; 
				break;

			case DetectionMiss: 
				count = ++mHistogramMisses[i][EHistogramTypeIndexLukas]; 
				if (mHistogramMaxCount[EHistogramTypeIndexLukas] < count)
				{
					mHistogramMaxCount[EHistogramTypeIndexLukas] = count;
				}
				break;

			case DetectionFalseAlarm: 
				count = ++mHistogramFalseAlarms[i][EHistogramTypeIndexLukas]; 
				break;

			case DetectionUnknown: 
				break;
		}
	}
}

void HistogramPlot::PostProcessData(IThreadStateCallback* pThread)
{
	if (pThread)
	{
		if (pThread->GetStateAbort() || pThread->GetStateRestart())
		{
			DBG("HistogramPlot::PostProcessData(): abort || restart");
			return;
		}
	}
/*
	unsigned int size = mHistogramHits.size();
	for (unsigned int i=0; i<size-1; i++)
	{
		if (pThread)
		{
			if (pThread->GetStateAbort() || pThread->GetStateRestart())
			{
				return;
			}
		}
		mHistogramHits[i+1] += mHistogramHits[i];
		if (mHistogramMaxCount < mHistogramHits[i+1])
		{
			mHistogramMaxCount = mHistogramHits[i+1];
		}

		mHistogramMisses[size-2-i] += mHistogramMisses[size-1-i];
		if (mHistogramMaxCount < mHistogramMisses[size-2-i])
		{
			mHistogramMaxCount = mHistogramMisses[size-2-i];
		}

		mHistogramFalseAlarms[size-2-i] += mHistogramFalseAlarms[size-1-i];
		if (mHistogramMaxCount < mHistogramFalseAlarms[size-2-i])
		{
			mHistogramMaxCount = mHistogramFalseAlarms[size-2-i];
		}
	}
*/
	mReady = true;
	update();
}

float HistogramPlot::IndexToScore(EHistogramTypeIndex typeIndex, int i) const
{
	assert(typeIndex < EHistogramTypeIndexCOUNT);
	return mXMin[typeIndex] + i*mHistogramItemWidth[typeIndex];
}

int HistogramPlot::GetHistogramIndex(EHistogramTypeIndex typeIndex, double score)
{
	if (mHistogramItemWidth[typeIndex] == 0)
	{
		Q_ASSERT(mHistogramHits.size() > 0);
		return 0;
	}
	else if (score <= mXMin[typeIndex])
	{
		return 0;
	}
	else if (score >= mXMax[typeIndex])
	{
		return mHistogramHits.size()-1;
	}
	else
	{
		Q_ASSERT_X(mHistogramHits.size() >= (int)((score - mXMin[typeIndex]) / mHistogramItemWidth[typeIndex]), "HistogramPlot::GetHistogramIndex()", QString("GetHistogramIndex(): score:%1 mXMin[typeIndex]:%2 mHistogramItemWidth[typeIndex]:%3").arg(score, mXMin[typeIndex], mHistogramItemWidth[typeIndex]).toStdString().c_str());
//		DBG("GetHistogramIndex(): typeIndex:"<<typeIndex<<" score:"<<score<<" mXMin:"<<mXMin[typeIndex]<<" mHistogramItemWidth:"<<mHistogramItemWidth[typeIndex]);
		return (int)((score - mXMin[typeIndex]) / mHistogramItemWidth[typeIndex]);
	}
}

void HistogramPlot::paintEvent(QPaintEvent * event)
{
	Q_UNUSED(event);
/*	
	if (!mReady)
	{
		return;
	}
*/
	QPainter painter(this);
	painter.save();

	QBrush brushNormal = painter.brush();

	QBrush brushHits;
	brushHits.setStyle(Qt::SolidPattern);
	brushHits.setColor(Qt::darkGreen);

	QBrush brushMisses;
	brushMisses.setStyle(Qt::SolidPattern);
	brushMisses.setColor(Qt::blue);

	QBrush brushFalseAlarms;
	brushFalseAlarms.setStyle(Qt::NoBrush);
	brushFalseAlarms.setColor(Qt::red);

//	painter.setBrush(brush);

	QPen penHits = painter.pen();
	penHits.setColor(Qt::darkGreen);

	QPen penMisses = painter.pen();
	penMisses.setColor(Qt::blue);

	QPen penFalseAlarms = painter.pen();
	penFalseAlarms.setColor(Qt::red);
	penFalseAlarms.setWidth(2);

	QPen penScores = painter.pen();
	penScores.setColor(Qt::darkGray);

	QPen penThreshold = painter.pen();
	penThreshold.setColor(Qt::black);
	penThreshold.setWidth(3);

	QPen penCursor = painter.pen();
	penCursor.setColor(Qt::red);
	penCursor.setWidth(1);

	QPen penBorder = painter.pen();
	penBorder.setColor(Qt::black);
	penBorder.setWidth(1);


//	painter.drawLine(10,10,100,100);

/*
	QPainterPath pathScores;
	CreatePath(&pathScores, &HistogramPlot::GetCountScores);
	painter.setPen(penScores);
	painter.drawPath(pathScores);
*/

	if (mReady)
	{
		for (int i=0; i<EHistogramTypeIndexCOUNT; i++)
		{
//			DBG("typeIndex:"<<i<<" mHistogramType:"<<(int)mHistogramType<<" TypeIndex2Type:"<<(int)TypeIndex2Type((EHistogramTypeIndex)i));
			if (mpPathMisses[i]) { delete mpPathMisses[i]; mpPathMisses[i] = NULL; }
			if (mpPathHits[i]) { delete mpPathHits[i]; mpPathHits[i] = NULL; }
			if (mpPathFalseAlarms[i]) { delete mpPathFalseAlarms[i]; mpPathFalseAlarms[i] = NULL; }

			if (mHistogramType & TypeIndex2Type((EHistogramTypeIndex)i))
			{
				mpPathMisses[i] = new QPainterPath;
				CreatePath(mpPathMisses[i], &HistogramPlot::GetCountMissesForIndex, (EHistogramTypeIndex)i, true);

				if (i == EHistogramTypeIndexIgiAccumulative || i == EHistogramTypeIndexLukas)
				{
					mpPathHits[i] = new QPainterPath; 
					CreatePath(mpPathHits[i], &HistogramPlot::GetCountHitsForIndex, (EHistogramTypeIndex)i, true);
				}

				mpPathFalseAlarms[i] = new QPainterPath; 
				CreatePath(mpPathFalseAlarms[i], &HistogramPlot::GetCountFalseAlarmsForIndex, (EHistogramTypeIndex)i, false);
			}
		}
	}
/*
	for (int i=0; i<mHistogramHits.size(); i++)
	{
		DBG("i:"<<i<<" Hits:"<<mHistogramHits[i]<<" Misses:"<<mHistogramMisses[i]<<" sum:"<<mHistogramHits[i]+mHistogramMisses[i]<<" thr:"<<IndexToScore(i));
//		DBG("stacked Hits:"<<GetCountHits(i)<<" Misses:"<<GetCountMisses(i));
	}
*/
	for (int i=0; i<EHistogramTypeIndexCOUNT; i++)
	{
//		DBG("typeIndex:"<<i);
		if (mpPathMisses[i])
		{
//			DBG("mpPathMisses");
			painter.setPen(penMisses);
			if (i == EHistogramTypeIndexIgiAccumulative)
			{
				painter.setBrush(brushMisses);
			}
			painter.drawPath(*(mpPathMisses[i]));
		}

		if (mpPathHits[i])
		{
//			DBG("mpPathHits");
			painter.setPen(penHits);
			if (i == EHistogramTypeIndexIgiAccumulative)
			{
				painter.setBrush(brushHits);
			}
			painter.drawPath(*(mpPathHits[i]));
		}

		if (mpPathFalseAlarms[i])
		{
//			DBG("mpPathFalseAlarms");
			painter.setPen(penFalseAlarms);
			if (i == EHistogramTypeIndexIgiAccumulative)
			{
				painter.setBrush(brushFalseAlarms);
			}
			painter.drawPath(*(mpPathFalseAlarms[i]));
		}
	}

/*
	float bottom_space = (float)0.0;
	float norm_width = (float)1/mHistogramHits.size();
	float norm_height = (float)(1-bottom_space)/mHistogramMaxCount;

	QPainterPath pathHits;
	QPainterPath pathMissses;
	pathHits->moveTo(CoordX(0), CoordY(1));
	for (int i=0; i<mHistogramHits.size(); i++)
	{
//		qDebug() << "i:"<<i<<" count[i]:"<<CALL_MEMBER_FN(*this, fArray)(i)<<" norm_width:"<<norm_width<<" norm_height:"<<norm_height<<" ("<<norm_width * i<<","<<(1 - norm_height * CALL_MEMBER_FN(*this, fArray)(i))<<","<<norm_width * (i+1)<<","<<1 - norm_height * CALL_MEMBER_FN(*this, fArray)(i);

		path->lineTo(
				CoordX(norm_width * i),
				CoordY(1 - bottom_space - norm_height * CALL_MEMBER_FN(*this, fArray)(i))
				);
		path->lineTo(
				CoordX(norm_width * (i+1)),
				CoordY(1 - bottom_space - norm_height * CALL_MEMBER_FN(*this, fArray)(i))
				);
	}
	path->lineTo(CoordX(1), CoordY(1));
	path->lineTo(CoordX(0), CoordY(1));
	path->closeSubpath();
*/

	
	int typeIndex = GetTypeIndex();
	{
		float x_max = mXMax[typeIndex];
		float x_min = mXMin[typeIndex];
/*		if (typeIndex != EHistogramTypeIndexIgiAccumulative)
		{
			x_max = (x_max - mAdditiveConstant) * mMultiplicativeConstant;
			x_min = (x_min - mAdditiveConstant) * mMultiplicativeConstant;
		}
*/		painter.drawText(QRect(0,rect().height()-20,50,20), Qt::AlignLeft | Qt::AlignBottom,QString("%1").arg(x_min, 0, 'f', 4));
		painter.drawText(QRect(rect().width()-50,rect().height()-20,50,20), Qt::AlignRight | Qt::AlignBottom,QString("%1").arg(x_max, 0, 'f', 4));
	}

	float x_max = mXMax[typeIndex];
	float x_min = mXMin[typeIndex];
	if (mCursorShow)
	{
		painter.setPen(penCursor);
		float cur_x = (float)1/(x_max - x_min) * (mCursor - x_min);
		painter.drawLine(CoordX(cur_x), CoordY(0), CoordX(cur_x), CoordY(1));
	}
	else
	{
		painter.setPen(penThreshold);
		float thr_x = (float)1/(x_max - x_min) * (mThreshold - x_min);
		DBG("thr_x = " << (float)1/(x_max - x_min) << " * " << (mThreshold - (x_min<0 ? -x_min : x_min)) << " = " << thr_x);
		painter.drawLine(CoordX(thr_x), CoordY(0), CoordX(thr_x), CoordY(1));
	}

	{
		painter.setPen(penBorder);
		QRect r = rect().adjusted(0, 0, -1, -20);
		painter.drawRect(r);
	}

/*	
	unsigned int x = 0;
	unsigned int label_height = 20;
	unsigned int label_width = rect().width();
	painter.setPen(penHits);
	painter.drawText(QRect(x,rect().height()-4*label_height,label_width,label_height), Qt::AlignHCenter | Qt::AlignBottom, tr("Hits"));
	painter.setPen(penMisses);
	painter.drawText(QRect(x,rect().height()-3*label_height,label_width,label_height), Qt::AlignHCenter | Qt::AlignBottom, tr("Misses"));
	painter.setPen(penFalseAlarms);
	painter.drawText(QRect(x,rect().height()-2*label_height,label_width,label_height), Qt::AlignHCenter | Qt::AlignBottom, tr("False alarms"));
	painter.setPen(penScores);
	painter.drawText(QRect(x,rect().height()-1*label_height,label_width,label_height), Qt::AlignHCenter | Qt::AlignBottom, tr("All"));
*/
	painter.setBrush(brushNormal);
	painter.restore();
}

void HistogramPlot::CreatePath(QPainterPath* path, unsigned int (HistogramPlot::*fArray)(EHistogramTypeIndex typeIndex, int), EHistogramTypeIndex typeIndex, bool close)
{
	float bottom_space = (float)0.0;
	float norm_width = (float)1/mHistogramHits.size();
	float norm_height = (float)(1-bottom_space)/(mHistogramMaxCount[typeIndex] * mZoom);

	path->moveTo(CoordX(0), CoordY(1));
	for (int i=0; i<mHistogramHits.size(); i++)
	{
		path->lineTo(
				CoordX(norm_width * i),
				CoordY(1 - bottom_space - norm_height * CALL_MEMBER_FN(*this, fArray)(typeIndex, i))
				);
		path->lineTo(
				CoordX(norm_width * (i+1)),
				CoordY(1 - bottom_space - norm_height * CALL_MEMBER_FN(*this, fArray)(typeIndex, i))
				);
//		DBG("i:"<<i<<" misses:"<<mHistogramMisses[i][EHistogramTypeIndexLukasAccumulative]<<" fa:"<<mHistogramFalseAlarms[i][EHistogramTypeIndexLukasAccumulative]);
	}
//	int i = 25;
//	DBG("CreatePath fArray:"<<(int)fArray<<" 1 - "<<bottom_space<<" - "<<norm_height<<" * ");
//	DBG("CreatePath fArray: 1 - "<<bottom_space<<" - "<<norm_height<<" * "<<(CALL_MEMBER_FN(*this, fArray)(typeIndex, i))<<" = "<<(1 - bottom_space - norm_height * CALL_MEMBER_FN(*this, fArray)(typeIndex, i)));
//	DBG("misses[25]: "<<mHistogramMisses[25][EHistogramTypeIndexLukasAccumulative]<<" fa[25]: "<<mHistogramFalseAlarms[25][EHistogramTypeIndexLukasAccumulative]);
	if (close && typeIndex != EHistogramTypeIndexLukasAccumulative && typeIndex != EHistogramTypeIndexLukas)
	{
		path->lineTo(CoordX(1), CoordY(1));
		path->lineTo(CoordX(0), CoordY(1));
		path->closeSubpath();
	}
}

int HistogramPlot::CoordX(double x)
{
	return (int)((double)rect().width() * x);
}

int HistogramPlot::CoordY(double y)
{
	return (int)((double)(rect().height()-20) * y);
}

unsigned int HistogramPlot::GetCountHitsForIndex(EHistogramTypeIndex typeIndex, int i)
{
	switch (typeIndex)
	{
		case EHistogramTypeIndexIgiAccumulative:
			return (int)((float)mHistogramHits[i][typeIndex] / (mHistogramHits[i][typeIndex] + mHistogramMisses[i][typeIndex]) * mHistogramMaxCount[typeIndex]);

		case EHistogramTypeIndexLukasAccumulative:
		case EHistogramTypeIndexLukas:
			return mHistogramHits[i][typeIndex];
		case EHistogramTypeIndexCOUNT:
			return 0;
	}
	return 0;
//	return mHistogramFalseAlarms[i] + mHistogramHits[i];
}

unsigned int HistogramPlot::GetCountMissesForIndex(EHistogramTypeIndex typeIndex, int i)
{
	Q_UNUSED(i);
	switch (typeIndex)
	{
		case EHistogramTypeIndexIgiAccumulative:
			return mHistogramMaxCount[typeIndex];

		case EHistogramTypeIndexLukasAccumulative:
		case EHistogramTypeIndexLukas:
			return mHistogramMisses[i][typeIndex];
		case EHistogramTypeIndexCOUNT:
			return 0;
	}
	return 0;
}

unsigned int HistogramPlot::GetCountFalseAlarmsForIndex(EHistogramTypeIndex typeIndex, int i)
{
	switch (typeIndex)
	{
		case EHistogramTypeIndexIgiAccumulative:
			return (int)((float)(mHistogramFalseAlarms[i][typeIndex]) / (mHistogramHits[i][typeIndex] + mHistogramMisses[i][typeIndex]) * mHistogramMaxCount[typeIndex]);

		case EHistogramTypeIndexLukasAccumulative:
		case EHistogramTypeIndexLukas:
			return mHistogramFalseAlarms[i][typeIndex];
		case EHistogramTypeIndexCOUNT:
			return 0;
	}
	return 0;
}

unsigned int HistogramPlot::GetCountHits(EHistogramTypeIndex typeIndex)
{ 
//	DBG("mThreshold:"<<mThreshold<<" index:"<<GetHistogramIndex(typeIndex, mThreshold));
	int i = GetHistogramIndex(typeIndex, mCursorShow ? mCursor : mThreshold);
	if (0 <= i && i < mHistogramHits.size())
	{
		return mHistogramHits[i][typeIndex];
	}
	else
	{
		return 0;
	}
}
	
unsigned int HistogramPlot::GetCountMisses(EHistogramTypeIndex typeIndex) 
{ 
	int i = GetHistogramIndex(typeIndex, mCursorShow ? mCursor : mThreshold);
	if (0 <= i && i < mHistogramMisses.size())
	{
		return mHistogramMisses[i][typeIndex]; 
	}
	else
	{
		return 0;
	}
}

unsigned int HistogramPlot::GetCountFalseAlarms(EHistogramTypeIndex typeIndex) 
{ 
	int i = GetHistogramIndex(typeIndex, mCursorShow ? mCursor : mThreshold);
	DBG("GetCountFalseAlarms(): typeIndex:"<<typeIndex<<" i:"<<i<<" fa:"<<mHistogramFalseAlarms[i][typeIndex]);
	if (0 <= i && i < mHistogramFalseAlarms.size())
	{
		return mHistogramFalseAlarms[i][typeIndex]; 
	}
	else
	{
		return 0;
	}
}

unsigned int HistogramPlot::GetCountHits()
{
	return GetCountHits(GetTypeIndex());
}

unsigned int HistogramPlot::GetCountMisses()
{
	return GetCountMisses(GetTypeIndex());
}

unsigned int HistogramPlot::GetCountFalseAlarms()
{
	return GetCountFalseAlarms(GetTypeIndex());
}

void HistogramPlot::SetFixedScoreInterval(EHistogramTypeIndex typeIndex, bool fixed, double xMin, double xMax)
{
	mFixedScoreInterval = fixed;
	if (mFixedScoreInterval)
	{
		mXMin[typeIndex] = xMin;
		mXMax[typeIndex] = xMax;
	}
}

void HistogramPlot::mouseMoveEvent (QMouseEvent * event)
{
	int typeIndex = GetTypeIndex();
	DBG("mouseMoveEvent: "<<event->pos().x()<<", "<<event->pos().y());
	mCursorShow = true;
	mCursor = mXMin[typeIndex] + ((float)event->pos().x() / width()) * (mXMax[typeIndex] - mXMin[typeIndex]);
	DBG("mCursor:"<<mCursor);
	update();
	emit StatisticsChanged();
}

void HistogramPlot::leaveEvent (QEvent * event)
{
	Q_UNUSED(event);
	DBG("leaveEvent");
	mCursorShow = false;
	update();
	emit StatisticsChanged();
}

void HistogramPlot::wheelEvent (QWheelEvent * event)
{
	int numDegrees = event->delta() / 8;
	int numSteps = numDegrees / 15;
	DBG("numDegrees:"<<numDegrees<<" numSteps:"<<numSteps);
	mZoom *= numSteps > 0 ? 1.1 : 0.9;
/*	
	unsigned int maxCountPrev = mHistogramMaxCount[GetTypeIndex()];
	mHistogramMaxCount[GetTypeIndex()] = mHistogramMaxCount[GetTypeIndex()] * (numSteps > 0 ? 1.1 : 0.9);
	if (maxCountPrev == mHistogramMaxCount[GetTypeIndex()])
	{
		mHistogramMaxCount[GetTypeIndex()] += numSteps;
	}
	DBG("mHistogramMaxCount:"<<mHistogramMaxCount[GetTypeIndex()]);
*/	update();
}

void HistogramPlot::SetHistogramType(unsigned char histogramType)
{
	DBG("SetHistogramType");
	mHistogramType = histogramType;
	if (histogramType & EHistogramTypeIgiAccumulative)
	{
		DBG("EHistogramTypeIgiAccumulative");
	}
	if (histogramType & EHistogramTypeLukasAccumulative)
	{
		DBG("EHistogramTypeLukasAccumulative");
	}
	if (histogramType & EHistogramTypeLukas)
	{
		DBG("EHistogramTypeLukas");
	}
}

unsigned char HistogramPlot::TypeIndex2Type(EHistogramTypeIndex typeIndex)
{
	switch (typeIndex)
	{
		case EHistogramTypeIndexIgiAccumulative:
			return EHistogramTypeIgiAccumulative;

		case EHistogramTypeIndexLukasAccumulative:
			return EHistogramTypeLukasAccumulative;

		case EHistogramTypeIndexLukas:
			return EHistogramTypeLukas;

		case EHistogramTypeIndexCOUNT:
			return 0;
	}
	return 0;
}

HistogramPlot::EHistogramTypeIndex HistogramPlot::GetTypeIndex()
{
	EHistogramTypeIndex typeIndex = EHistogramTypeIndexIgiAccumulative;
	if (mHistogramType & EHistogramTypeLukasAccumulative)
	{
		typeIndex = EHistogramTypeIndexLukasAccumulative;
	}
	else if (mHistogramType & EHistogramTypeLukas)
	{
		typeIndex = EHistogramTypeIndexLukas;
	}
	return typeIndex;
}

