#ifndef HISTOGRAMPLOT_H
#define HISTOGRAMPLOT_H

#include <QWidget>
#include <QFrame>
#include <QPainterPath>

#include "ithreadstatecallback.h"
#include "mlfmodeldata.h"
#include "dbg.h"

class HistogramPlot : public QFrame
{
	Q_OBJECT

	static const unsigned int mHistogramItemsCount = 50;

	public:
		enum EDetectionType { DetectionHit, DetectionMiss, DetectionFalseAlarm, DetectionUnknown };
		enum EHistogramType { EHistogramTypeIgiAccumulative = 0x1, EHistogramTypeLukasAccumulative = 0x2, EHistogramTypeLukas = 0x4 };
		enum EHistogramTypeIndex { EHistogramTypeIndexIgiAccumulative, EHistogramTypeIndexLukasAccumulative, EHistogramTypeIndexLukas, EHistogramTypeIndexCOUNT };

		HistogramPlot(QWidget* parent = NULL);
		void Reset(double xMin, double xMax);
		void AddData(MlfModelData::MlfTableRow * pRow);
		void PostProcessData(IThreadStateCallback* pThread = 0);
		void SetThreshold(double v) { mThreshold = v; update(); }
		void SetFixedScoreInterval(EHistogramTypeIndex typeIndex, bool fixed, double xMin = 0, double xMax = 0);
		void SetReferenceMisses(unsigned int count) { DBG("SetReferenceMisses("<<count<<") this:"<<(int)this); mReferenceMisses = count; }
		unsigned int GetReferenceMisses() { return mReferenceMisses; }
		void SetHistogramType(unsigned char histogramType);
		unsigned char GetHistogramType() { return mHistogramType; }
		void SetAdditiveConstant(double additive) { mAdditiveConstant = additive; }
		void SetMultiplicativeConstant(double multiplicative) { mMultiplicativeConstant = multiplicative; }

//		unsigned int GetHistogramMaxCount() { return mHistogramMaxCount; }
//		float GetHistogramItemWidth() { return mHistogramItemWidth; }

//		const QVector<unsigned int>& GetHistogramScores() { return mHistogramScores; }
		unsigned int GetCountHits(EHistogramTypeIndex typeIndex);
		unsigned int GetCountMisses(EHistogramTypeIndex typeIndex);
		unsigned int GetCountFalseAlarms(EHistogramTypeIndex typeIndex);

		unsigned int GetCountHits();
		unsigned int GetCountMisses();
		unsigned int GetCountFalseAlarms();

	signals:
		void StatisticsChanged();

	protected:
		void paintEvent(QPaintEvent *event);
		void mouseMoveEvent (QMouseEvent * event);
		void leaveEvent (QEvent * event);
		void wheelEvent (QWheelEvent * event);

	private:
		double					mThreshold;
		double					mCursor;
		bool					mCursorShow;
		bool					mFixedScoreInterval;
		double					mXMin[EHistogramTypeIndexCOUNT];
		double					mXMax[EHistogramTypeIndexCOUNT];
		float 					mHistogramItemWidth[EHistogramTypeIndexCOUNT];
		unsigned int			mReferenceMisses;
		double 					mAdditiveConstant;
		double 					mMultiplicativeConstant;
		double					mZoom;

//		QVector<unsigned int>	mHistogramScores;
		QVector< QVector<unsigned int> >	mHistogramHits;
		QVector< QVector<unsigned int> >	mHistogramMisses;
		QVector< QVector<unsigned int> >	mHistogramFalseAlarms;




		unsigned int			mHistogramMaxCount[EHistogramTypeIndexCOUNT];
		bool					mReady;
		QPainterPath *			mpPathHits[EHistogramTypeIndexCOUNT];
		QPainterPath *			mpPathMisses[EHistogramTypeIndexCOUNT];
		QPainterPath *			mpPathFalseAlarms[EHistogramTypeIndexCOUNT];

		unsigned char			mHistogramType;

		int GetHistogramIndex(EHistogramTypeIndex typeIndex, double score);
		float IndexToScore(EHistogramTypeIndex typeIndex, int i) const;
		int CoordX(double x);
		int CoordY(double y);
		void CreatePath(QPainterPath* path, unsigned int (HistogramPlot::*fArray)(EHistogramTypeIndex typeIndex, int), EHistogramTypeIndex typeIndex, bool close);

		unsigned int GetCountHitsForIndex(EHistogramTypeIndex typeIndex, int i);
		unsigned int GetCountMissesForIndex(EHistogramTypeIndex typeIndex, int i);
		unsigned int GetCountFalseAlarmsForIndex(EHistogramTypeIndex typeIndex, int i);
//		unsigned int GetCountScores(int i);
		EDetectionType GetRowType(MlfModelData::ECorrect correct, double score, EHistogramTypeIndex typeIndex, int histogramIndex) const;
		unsigned char TypeIndex2Type(EHistogramTypeIndex typeIndex);
		EHistogramTypeIndex GetTypeIndex();
};

#endif

