#include "keywordsfilter.h"
#include <QtGui>


KeywordsFilter::KeywordsFilter(QWidget* parent) : QWidget(parent)
{
	// WIDGETS
	mpModelDeselected = new QStandardItemModel;
	mpListKeywordsDeselected = new QListView;
	mpListKeywordsDeselected->setModel(mpModelDeselected);
	mpListKeywordsDeselected->setEditTriggers(QAbstractItemView::NoEditTriggers);

	mpModelSelected = new QStandardItemModel;
	mpListKeywordsSelected = new QListView;
	mpListKeywordsSelected->setModel(mpModelSelected);
	mpListKeywordsSelected->setEditTriggers(QAbstractItemView::NoEditTriggers);

	QPushButton * buttonKeywordSelect = new QPushButton(">");
	QPushButton * buttonKeywordDeselect = new QPushButton("<");
	QPushButton * buttonKeywordSelectAll = new QPushButton(">>");
	QPushButton * buttonKeywordDeselectAll = new QPushButton("<<");
	QLabel * labelKeywordsDeselected = new QLabel(tr("Hidden keywords"));
	QLabel * labelKeywordsSelected = new QLabel(tr("Selected keywords"));
	QPushButton * buttonOk = new QPushButton(style()->standardIcon(QStyle::SP_DialogOkButton), tr("Ok"));
	QPushButton * buttonCancel = new QPushButton(style()->standardIcon(QStyle::SP_DialogCancelButton), tr("Cancel"));

	// CONNECTIONS
	connect(buttonKeywordSelect, SIGNAL(clicked()), this, SLOT(SelectActiveKeyword()));
	connect(buttonKeywordDeselect, SIGNAL(clicked()), this, SLOT(DeselectActiveKeyword()));
	connect(buttonKeywordSelectAll, SIGNAL(clicked()), this, SLOT(SelectAllKeywords()));
	connect(buttonKeywordDeselectAll, SIGNAL(clicked()), this, SLOT(DeselectAllKeywords()));
	connect(mpListKeywordsDeselected, SIGNAL(doubleClicked(const QModelIndex &)), this, SLOT(SelectActiveKeyword()));
	connect(mpListKeywordsSelected, SIGNAL(doubleClicked(const QModelIndex &)), this, SLOT(DeselectActiveKeyword()));
	connect(buttonOk, SIGNAL(clicked()), this, SLOT(OkClicked()));
	connect(buttonCancel, SIGNAL(clicked()), this, SLOT(CancelClicked()));
	

	// LAYOUT
	QVBoxLayout * layoutButtons = new QVBoxLayout;
	layoutButtons->addStretch();
	layoutButtons->addWidget(buttonKeywordSelect);
	layoutButtons->addWidget(buttonKeywordDeselect);
	layoutButtons->addSpacing(20);
	layoutButtons->addWidget(buttonKeywordSelectAll);
	layoutButtons->addWidget(buttonKeywordDeselectAll);
	layoutButtons->addStretch();

	QVBoxLayout * layoutKeywordsDeselected = new QVBoxLayout;
	layoutKeywordsDeselected->addWidget(labelKeywordsDeselected);
	layoutKeywordsDeselected->addWidget(mpListKeywordsDeselected);

	QVBoxLayout * layoutKeywordsSelected = new QVBoxLayout;
	layoutKeywordsSelected->addWidget(labelKeywordsSelected);
	layoutKeywordsSelected->addWidget(mpListKeywordsSelected);

	QHBoxLayout * layoutLists = new QHBoxLayout;
	layoutLists->addLayout(layoutKeywordsDeselected);
	layoutLists->addLayout(layoutButtons);
	layoutLists->addLayout(layoutKeywordsSelected);

	QHBoxLayout * layoutOkCancel = new QHBoxLayout;
	layoutOkCancel->addStretch();
	layoutOkCancel->addWidget(buttonOk);
	layoutOkCancel->addWidget(buttonCancel);
	layoutOkCancel->addStretch();

	QVBoxLayout * layoutMain = new QVBoxLayout;
	layoutMain->addLayout(layoutLists);
	layoutMain->addSpacing(20);
	layoutMain->addLayout(layoutOkCancel);

	setLayout(layoutMain);
	setFocusPolicy(Qt::StrongFocus);
	hide();
}

void KeywordsFilter::SelectActiveKeyword()
{
	QModelIndexList list = mpListKeywordsDeselected->selectionModel()->selectedRows();
	for (QModelIndexList::iterator i=list.begin(); i!=list.end(); i++)
	{
		SelectKeyword(i->row());
	}
//	SelectKeyword(mActiveRowDeselected);
}

void KeywordsFilter::DeselectActiveKeyword()
{
	QModelIndexList list = mpListKeywordsSelected->selectionModel()->selectedRows();
	for (QModelIndexList::iterator i=list.begin(); i!=list.end(); i++)
	{
		DeselectKeyword(i->row());
	}
//	DeselectKeyword(mActiveRowSelected);
}

void KeywordsFilter::SelectKeyword(int row)
{
	DBG("KeywordsFilter::SelectKeyword("<<row<<")");
	QModelIndex i = mpModelDeselected->index(row, 0);
	QString keyword =  i.data().toString();
	(*mpMap)[keyword] = true;
	mpModelDeselected->removeRow(row);
	QStandardItem * item = new QStandardItem(keyword);
	mpModelSelected->appendRow(item);
}

void KeywordsFilter::DeselectKeyword(int row)
{
	DBG("KeywordsFilter::DeselectKeyword("<<row<<")");
	QModelIndex i = mpModelSelected->index(row, 0);
	QString keyword = i.data().toString();
	(*mpMap)[keyword] = false;
	mpModelSelected->removeRow(row);
	QStandardItem * item = new QStandardItem(keyword);
	mpModelDeselected->appendRow(item);
}

void KeywordsFilter::SelectAllKeywords()
{
	int count = mpModelDeselected->rowCount();
	for (int i=0; i<count; i++)
	{
		SelectKeyword(0);
	}
	mpModelSelected->sort(0);
}

void KeywordsFilter::DeselectAllKeywords()
{
	int count = mpModelSelected->rowCount();
	for (int i=0; i<count; i++)
	{
		DeselectKeyword(0);
	}
	mpModelDeselected->sort(0);
}

void KeywordsFilter::show()
{
	mpModelSelected->clear();
	mpModelDeselected->clear();
	for (MlfSortingThread::SelectedKeywordsMap::iterator i=mpMap->begin(); i!=mpMap->end(); i++)
	{
		DBG("key:"<<i.key().toStdString());
		QStandardItem * item = new QStandardItem(i.key());
		if (i.value())
		{
			mpModelSelected->appendRow(item);
		}
		else 
		{
			mpModelDeselected->appendRow(item);
		}
	}
	mpModelSelected->sort(0);
	mpModelDeselected->sort(0);
	(static_cast<QWidget*>(this))->show();
}

/*
void KeywordsFilter::SetMlfModelData(const MlfModelData::Data* data)
{
	for (MlfModelData::Data::const_iterator i=data->begin(); i!=data->end(); i++)
	{
		MlfModelData::MlfTableRow * row = *i;
		
		(*mpMap)[row->mLabelUnicode] = true;
		QStandardItem * item = new QStandardItem(row->mLabelUnicode);
		mpModelSelected->appendRow(item);
	}
	mpModelSelected->sort(0);
}
*/

void KeywordsFilter::ListDeselectedClicked(const QModelIndex & index)
{
	mActiveRowDeselected = index.row();
}

void KeywordsFilter::ListSelectedClicked(const QModelIndex & index)
{
	mActiveRowSelected = index.row();
}

void KeywordsFilter::OkClicked()
{
	emit FilterChanged();
	close();
}

void KeywordsFilter::CancelClicked()
{
	close();
}

void KeywordsFilter::keyPressEvent(QKeyEvent * event)
{
	switch(event->key())
	{
		case Qt::Key_Escape: CancelClicked(); break;
	}
}

