#include <QtGui>
#include "mainwindow.h"

#include <iostream>
#include <cmath>
using namespace std;

#ifdef DEBUG
//	#define PREDEFINED_PATHS
#endif
//#define SHOW_LLR_OPTION
//#define SHOW_PRIOR
//#define SHOW_HD1_ONLY
//#define SHOW_THRESHOLD

MainWindow::MainWindow(QSettings * pSettings, QWidget *parent)
	: QMainWindow(parent), mpSettings(pSettings)
{
	mThresholdValue = 0.5;
	mPriorValue = 0;
	mMultiplicativeValue = 0;
	mAdditiveValue = 0;

	mContextMsPrev = 0;
	mReferenceBoundaryMsPrev = 0;

	mpSelectedIndex = new QModelIndex;

	QLocale::setDefault(QLocale(QLocale::Czech, QLocale::CzechRepublic));

	//--------------------------------------------------
	// WINDOWS AND DIALOGS
	mpKeywordsFilter = new KeywordsFilter;

	mpOpenDialog = new OpenDialog;
	
	mpSettingsDialog = new SettingsDialog;

	mpAboutWindow = new AboutWindow;

	mpTutorialWindow = new TutorialWindow;

	//--------------------------------------------------
	// ACTIONS
	mpActionCopyToClipboard = new QAction(tr("&Copy to clipboard"), this);
	mpActionCopyToClipboard->setShortcut(tr("Ctrl+C"));
	connect(mpActionCopyToClipboard, SIGNAL(triggered()), this, SLOT(CopyToClipboard()));

	QAction * pactionopen = new QAction(tr("&Open"), this);
	pactionopen->setShortcut(tr("Ctrl+O"));
	connect(pactionopen, SIGNAL(triggered()), this, SLOT(Open()));

	QAction * pactionsavemlf = new QAction(tr("&Save MLF"), this);
	pactionsavemlf->setShortcut(tr("Ctrl+S"));
	connect(pactionsavemlf, SIGNAL(triggered()), this, SLOT(SaveMlf()));

	QAction * pactionsaverefmlf = new QAction(tr("&Save reference MLF"), this);
	pactionsaverefmlf->setShortcut(tr("Ctrl+R"));
	connect(pactionsaverefmlf, SIGNAL(triggered()), this, SLOT(SaveRefMlf()));

	QAction * pactionsettings = new QAction(tr("&Preferences"), this);
	pactionsettings->setShortcut(tr("Ctrl+P"));
	connect(pactionsettings, SIGNAL(triggered()), this, SLOT(Settings()));

	QAction * pactionkeywordsfilter = new QAction(tr("Keywords &filter"), this);
	pactionkeywordsfilter->setShortcut(tr("Ctrl+F"));
	connect(pactionkeywordsfilter, SIGNAL(triggered()), this, SLOT(ShowKeywordsFilter()));

	QAction * pactionexit = new QAction(tr("E&xit"), this);
//	pactionexit->setShortcut(tr("Ctrl+X"));
	connect(pactionexit, SIGNAL(triggered()), qApp, SLOT(quit()));

	QAction * pactiontutorial = new QAction(tr("&Help"), this);
	//pactiontutorial->setShortcut(tr("Ctrl+H"));
	connect(pactiontutorial, SIGNAL(triggered()), mpTutorialWindow, SLOT(show()));

	QAction * pactionabout = new QAction(tr("&About"), this);
	//pactionabout->setShortcut(tr("Ctrl+A"));
	connect(pactionabout, SIGNAL(triggered()), mpAboutWindow, SLOT(show()));

	//--------------------------------------------------
	// MENUS
	QMenu * fileMenu = menuBar()->addMenu(tr("&File"));
	fileMenu->addAction(pactionopen);
	fileMenu->addAction(pactionsavemlf);
	fileMenu->addAction(pactionsaverefmlf);
	fileMenu->addSeparator();
	fileMenu->addAction(pactionexit);

	QMenu * settingsMenu = menuBar()->addMenu(tr("&Settings"));
	settingsMenu->addAction(pactionkeywordsfilter);
	settingsMenu->addAction(pactionsettings);

	QMenu * helpMenu = menuBar()->addMenu(tr("&Help"));
	helpMenu->addAction(pactiontutorial);
	helpMenu->addAction(pactionabout);

	//--------------------------------------------------
	// COMPONENTS
	QWidget * centralWidget = new QWidget;
	setCentralWidget(centralWidget);

	mpClipboard = QApplication::clipboard();

	mpMlfModel = new MlfModel;
	mpMlfModel->SetProgressDisplay(this);
//	mpProxyModel = new QSortFilterProxyModel;
	mpProxyModel = new MlfSortFilterProxyModel(mpSettings);
	mpProxyModel->SetProgressDisplay(this);
	mpProxyModel->setSourceModel(mpMlfModel);	
//	mpProxyModel->setDynamicSortFilter(true);
	mpKeywordsFilter->SetSelectedKeywordsMap(mpProxyModel->GetSelectedKeywordsMap());

	mpTableView = new QTableView;
	mpTableView->setSelectionBehavior(QAbstractItemView::SelectRows);
	mpTableView->setSortingEnabled(true);
	mpTableView->setContextMenuPolicy(Qt::CustomContextMenu);
	//mpTableView->setStyleSheet("selection-background-color: transparent;");
	mpTableView->setStyleSheet("QTableView { selection-background-color: #7777ff; }");
	mpTableView->setShowGrid(false);
	mpTableView->verticalHeader()->setDefaultSectionSize(20);

	mpTableItemDelegate = new TableItemDelegate(this);
	mpTableItemDelegate->SetSourceModel(mpMlfModel);
	mpTableItemDelegate->SetProxyModel(mpProxyModel);
	mpTableItemDelegate->SetColumns(mpProxyModel->GetColumns());
	mpTableView->setItemDelegate(mpTableItemDelegate);
	mpTableItemDelegate->SetTableView(mpTableView);
	mpTableItemDelegate->SetFixedScoreInterval(true, 0, 1);

	mpHistogramPlot = new HistogramPlot;
	mpHistogramPlot->setFrameStyle(QFrame::Panel | QFrame::Sunken);
	mpHistogramPlot->setLineWidth(2);
	mpHistogramPlot->SetFixedScoreInterval(HistogramPlot::EHistogramTypeIndexIgiAccumulative, true, 0.0, 1.0);
	mpHistogramPlot->SetHistogramType(HistogramPlot::EHistogramTypeLukasAccumulative);
	mpMlfModel->SetHistogramPlot(mpHistogramPlot);

	mpMediaPlayer = new MediaPlayer();
	mpMediaPlayer->setEnabled(false);
//	mpMediaPlayer->hide();

#ifdef SHOW_HD1_ONLY
	mpCheckboxShowOnlyHD1 = new QCheckBox(tr("Show only HD=1"));
	mpCheckboxShowOnlyHD1->setToolTip(tr("Shows only those detections which normalized scores are above the threshold"));
#endif
/*
	mpButtonOpenMlf = new QPushButton(tr("&Open MLF"));
	mpButtonOpenRefMlf = new QPushButton(tr("Open &reference MLF"));
	mpButtonSaveMlf = new QPushButton(tr("&Save MLF"));
	mpButtonSaveRefMlf = new QPushButton(tr("S&ave reference MLF"));
	mpButtonSelectAudioFilesDirectory = new QPushButton(tr("Set &wavs folder..."));
	mpButtonKeywordsFilter = new QPushButton(tr("Keywords &filter..."));
	mpButtonQuit = new QPushButton(tr("&Quit"));
*/
#ifdef SHOW_PRIOR
	create_slider_spinbox_pair(&mpSliderPrior, &mpSpinboxPrior);
	set_slider_spinbox_pair(mpSliderPrior, mpSpinboxPrior, mPriorMin, mPriorMax, mPriorPrecision, mPriorDefaultValue, &mPriorValue);
#endif
#ifdef SHOW_LLR_OPTION	
	mpRadiobuttonLLR = new QRadioButton(tr("Log-likelihood ratio"));
	mpRadiobuttonKS = new QRadioButton(tr("0..1-score"));
#endif

	create_slider_spinbox_pair(&mpSliderMultiplicative, &mpSpinboxMultiplicative);
	set_slider_spinbox_pair(mpSliderMultiplicative, mpSpinboxMultiplicative, mMultiplicativeMin, mMultiplicativeMax, mMultiplicativePrecision, mMultiplicativeDefaultValue, &mMultiplicativeValue);

	create_slider_spinbox_pair(&mpSliderAdditive, &mpSpinboxAdditive);
	set_slider_spinbox_pair(mpSliderAdditive, mpSpinboxAdditive, mAdditiveMin, mAdditiveMax, mAdditivePrecision, mAdditiveDefaultValue, &mAdditiveValue);
	
#ifdef SHOW_THRESHOLD
	create_slider_spinbox_pair(&mpSliderThreshold, &mpSpinboxThreshold);
	set_slider_spinbox_pair(mpSliderThreshold, mpSpinboxThreshold, mThresholdMin, mThresholdMax, mThresholdPrecision, mThresholdDefaultValue, &mThresholdValue);
#endif

//	QStatusBar * mpStatusBar = new QStatusBar();
//	mpStatusBar->showMessage(tr("Ready"));

	mpProgressbarProcessing = new QProgressBar;
	mpProgressbarProcessing->setRange(0,100);
	mpProgressbarProcessing->setValue(0);
	mpProgressbarProcessing->setMaximumWidth(300);
	mpLabelProcessing = new QLabel;
	mpLabelProcessing->setText(tr("Sorting..."));
//	mpStatusBar->addWidget(mpLabelProcessing);
//	mpStatusBar->addWidget(mpProgressbarProcessing);

	QHBoxLayout * layoutStatusbar = new QHBoxLayout;
//	layoutStatusbar->addWidget(mpStatusBar);
	layoutStatusbar->addWidget(mpLabelProcessing);
	layoutStatusbar->addSpacing(20);
	layoutStatusbar->addWidget(mpProgressbarProcessing);
	layoutStatusbar->addStretch();
	layoutStatusbar->setSpacing(0);
	mpFrameProcessing = new QFrame;
	mpFrameProcessing->setFrameStyle(QFrame::Panel | QFrame::Sunken);
	mpFrameProcessing->setLineWidth(2);
	mpFrameProcessing->setLayout(layoutStatusbar);
	mpFrameProcessing->setMaximumHeight(35);
	mpFrameProcessing->hide();

	mpLabelHistogramHits = new QLabel(tr("Hits"));
	mpLabelHistogramHits->setStyleSheet("color: darkGreen");
	mpLabelHistogramMisses = new QLabel(tr("Misses"));
	mpLabelHistogramMisses->setStyleSheet("color: blue");
	mpLabelHistogramFalseAlarms = new QLabel(tr("False alarms"));
	mpLabelHistogramFalseAlarms->setStyleSheet("color: red");

//	mpProgressbarProcessing->setFormat("%p%");

	//--------------------------------------------------
	// SIGNAL CONNECTIONS
	connect(mpTableView, SIGNAL(doubleClicked(const QModelIndex &)), this, SLOT(TableItemDoubleClicked(const QModelIndex &)));
	connect(mpTableView, SIGNAL(clicked(const QModelIndex &)), this, SLOT(TableItemClicked(const QModelIndex &)));
	connect(mpTableView, SIGNAL(customContextMenuRequested (const QPoint & )), this, SLOT(TableContextMenuRequested(const QPoint &)));
#ifdef SHOW_HD1_ONLY	
	connect(mpCheckboxShowOnlyHD1, SIGNAL(toggled(bool)), this, SLOT(CheckboxShowOnlyHD1Toggled(bool)));
#endif
/*
	connect(mpButtonOpenMlf, SIGNAL(clicked()), this, SLOT(OpenMlf()));
	connect(mpButtonOpenRefMlf, SIGNAL(clicked()), this, SLOT(OpenRefMlf()));
	connect(mpButtonSaveMlf, SIGNAL(clicked()), this, SLOT(SaveMlf()));
	connect(mpButtonSaveRefMlf, SIGNAL(clicked()), this, SLOT(SaveRefMlf()));
	connect(mpButtonSelectAudioFilesDirectory, SIGNAL(clicked()), this, SLOT(SelectAudioFilesDirectory()));
	connect(mpButtonKeywordsFilter, SIGNAL(clicked()), this, SLOT(ShowKeywordsFilter()));
	connect(mpButtonQuit, SIGNAL(clicked()), qApp, SLOT(quit()));
*/	
#ifdef SHOW_THRESHOLD
	connect(mpSliderThreshold, SIGNAL(valueChanged(int)), this, SLOT(ThresholdSliderChanged(int)));
	connect(mpSpinboxThreshold, SIGNAL(valueChanged(double)), this, SLOT(ThresholdSpinboxChanged(double)));
#endif
#ifdef SHOW_PRIOR
	connect(mpSliderPrior, SIGNAL(valueChanged(int)), this, SLOT(PriorSliderChanged(int)));
	connect(mpSpinboxPrior, SIGNAL(valueChanged(double)), this, SLOT(PriorSpinboxChanged(double)));
#endif
#ifdef SHOW_LLR_OPTION
	connect(mpRadiobuttonLLR, SIGNAL(toggled(bool)), this, SLOT(RadiobuttonLLRToggled(bool)));
	connect(mpRadiobuttonKS, SIGNAL(toggled(bool)), this, SLOT(RadiobuttonKSToggled(bool)));
#endif
	connect(mpSliderMultiplicative, SIGNAL(valueChanged(int)), this, SLOT(MultiplicativeSliderChanged(int)));
	connect(mpSpinboxMultiplicative, SIGNAL(valueChanged(double)), this, SLOT(MultiplicativeSpinboxChanged(double)));
	connect(mpSliderAdditive, SIGNAL(valueChanged(int)), this, SLOT(AdditiveSliderChanged(int)));
	connect(mpSpinboxAdditive, SIGNAL(valueChanged(double)), this, SLOT(AdditiveSpinboxChanged(double)));
	connect(mpKeywordsFilter, SIGNAL(FilterChanged()), this, SLOT(FilterChanged()));
	connect(mpProxyModel, SIGNAL(dataChanged(const QModelIndex&, const QModelIndex&)), this, SLOT(OnDataChanged()));
	connect(mpHistogramPlot, SIGNAL(StatisticsChanged()), this, SLOT(OnHistogramStatisticsChanged()));
	connect(mpSettingsDialog, SIGNAL(RowHeightChanged(int)), this, SLOT(RowHeightChanged(int)));

	//--------------------------------------------------
	// LAYOUT
#ifdef SHOW_THRESHOLD
	QHBoxLayout *layoutThreshold = new QHBoxLayout;
	layoutThreshold->addWidget(mpSliderThreshold);
	layoutThreshold->addWidget(mpSpinboxThreshold);
	QGroupBox *groupboxThreshold = new QGroupBox(tr("Threshold"));
	groupboxThreshold->setLayout(layoutThreshold);
#endif
#ifdef SHOW_PRIOR
	QHBoxLayout *layoutPrior = new QHBoxLayout;
	layoutPrior->addWidget(mpSliderPrior);
	layoutPrior->addWidget(mpSpinboxPrior);
	QGroupBox *groupboxPrior = new QGroupBox(tr("Prior"));
	groupboxPrior->setLayout(layoutPrior);
#endif
#ifdef SHOW_LLR_OPTION
	QVBoxLayout *layoutScoretype = new QVBoxLayout;
	layoutScoretype->addWidget(mpRadiobuttonLLR);
	layoutScoretype->addWidget(mpRadiobuttonKS);
	QGroupBox *groupboxScoretype = new QGroupBox(tr("Normalization"));
	groupboxScoretype->setLayout(layoutScoretype);
#endif
	QHBoxLayout *layoutMultiplicative = new QHBoxLayout;
	layoutMultiplicative->addWidget(mpSliderMultiplicative);
	layoutMultiplicative->addWidget(mpSpinboxMultiplicative);
	mpGroupboxMultiplicative = new QGroupBox(tr("Multiplicative normalization constant"));
	mpGroupboxMultiplicative->setLayout(layoutMultiplicative);
	mpGroupboxMultiplicative->hide();

	QHBoxLayout *layoutAdditive = new QHBoxLayout;
	layoutAdditive->addWidget(mpSliderAdditive);
	layoutAdditive->addWidget(mpSpinboxAdditive);
	mpGroupboxAdditive = new QGroupBox(tr("Subtractive normalization constant"));
	mpGroupboxAdditive->setLayout(layoutAdditive);
	mpGroupboxAdditive->hide();

	QHBoxLayout *layoutHistogramLegend = new QHBoxLayout;
	layoutHistogramLegend->addWidget(mpLabelHistogramHits);
	layoutHistogramLegend->addWidget(mpLabelHistogramMisses);
	layoutHistogramLegend->addWidget(mpLabelHistogramFalseAlarms);
/*
	QVBoxLayout *layoutControlsMain = new QVBoxLayout;
	layoutControlsMain->addWidget(mpButtonOpenMlf);
	layoutControlsMain->addWidget(mpButtonOpenRefMlf);
	layoutControlsMain->addWidget(mpButtonSaveMlf);
	layoutControlsMain->addWidget(mpButtonSaveRefMlf);
	layoutControlsMain->addWidget(mpButtonSelectAudioFilesDirectory);
	layoutControlsMain->addWidget(mpButtonKeywordsFilter);
	layoutControlsMain->addWidget(mpButtonQuit);
	layoutControlsMain->addStretch();
*/
	QVBoxLayout *layoutControlsScore = new QVBoxLayout;
	layoutControlsScore->addWidget(mpMediaPlayer);
#ifdef SHOW_HD1_ONLY
	layoutControlsScore->addWidget(mpCheckboxShowOnlyHD1);
#endif
#ifdef SHOW_PRIOR
	layoutControlsScore->addWidget(groupboxPrior);
#endif
#ifdef SHOW_LLR_OPTION	
	layoutControlsScore->addWidget(groupboxScoretype);
#endif
	layoutControlsScore->addWidget(mpGroupboxMultiplicative);
	layoutControlsScore->addWidget(mpGroupboxAdditive);
#ifdef SHOW_THRESHOLD
	layoutControlsScore->addWidget(groupboxThreshold);
#endif
	layoutControlsScore->addWidget(mpHistogramPlot);
	layoutControlsScore->addLayout(layoutHistogramLegend);
	layoutControlsScore->addStretch();
/*
	QFrame *frameControlsMain = new QFrame;
	frameControlsMain->setLayout(layoutControlsMain);
*/	FrameControlsScore *frameControlsScore = new FrameControlsScore;
	frameControlsScore->setLayout(layoutControlsScore);
	connect(frameControlsScore, SIGNAL(OnResize(QResizeEvent *)), this, SLOT(FrameControlsResized(QResizeEvent *)));
/*
	QTabWidget *tabControls = new QTabWidget;
	tabControls->addTab(frameControlsMain, tr("Main"));
	tabControls->addTab(frameControlsScore, tr("Score"));
*/
	mpMainSplitter = new QSplitter(Qt::Horizontal);
	mpMainSplitter->addWidget(mpTableView);
	mpMainSplitter->addWidget(frameControlsScore);
	mpMainSplitter->setStretchFactor(0,3);
	mpMainSplitter->setStretchFactor(1,1);
/*
	QHBoxLayout *mpMainSplitter = new QHBoxLayout;
	mpMainSplitter->addWidget(mpTableView, 3);
	mpMainSplitter->addWidget(frameControlsMain, 1);
*/
	QVBoxLayout *mainLayout = new QVBoxLayout;
	mainLayout->addWidget(mpMainSplitter);
	mainLayout->addWidget(mpFrameProcessing);

//	mainLayout->addWidget(mpTableView, 3);
//	mainLayout->addLayout(layoutControls, 1);

	centralWidget->setLayout(mainLayout);
	setWindowTitle(tr("KWS Viewer"));
	qApp->setApplicationName("KwsViewer");

	mpTimerHideStatusbar = new QTimer(this);
	connect(mpTimerHideStatusbar, SIGNAL(timeout()), this, SLOT(HideStatusbar()));

	mpMlfModel->ThresholdChanged(mThresholdValue);
	mpTableItemDelegate->ThresholdChanged(mThresholdValue);

#ifdef SHOW_LLR_OPTION	
	mpRadiobuttonKS->setChecked(true);
#else
	RadiobuttonKSToggled(true);
#endif

	ReadSettings(mpSettings);
	//--------------------------------------------------
	// TESTING
//	OpenMlf();
//	OpenRefMlf();
}

void MainWindow::ReadSettings(QSettings * pSettings)
{
	mEncodingPrev = pSettings->value("encoding", "CP 1250").toString();
	mpSettingsDialog->SetEncoding(mEncodingPrev);
	mpMlfModel->SetEncoding(mEncodingPrev);

	mInputScoreTypePrev = static_cast<MlfModelData::EInputScoreType>(pSettings->value("input_score_type", 0).toUInt());
	mpSettingsDialog->SetInputScoreType(mInputScoreTypePrev);
	mpMlfModel->SetInputScoreType(mInputScoreTypePrev);

	mContextMsPrev = pSettings->value("context_ms", 0).toUInt();
	mpSettingsDialog->SetContextMilliseconds(mContextMsPrev);
	mpMediaPlayer->SetContextMilliseconds(mContextMsPrev);

	mAudioFileSuffixPrev = pSettings->value("audio_file_suffix", "wav").toString();
	mpSettingsDialog->SetAudioFileSuffix(mAudioFileSuffixPrev);
	mpMediaPlayer->SetAudioFileSuffix(mAudioFileSuffixPrev);

	mIsRawPrev = pSettings->value("is_raw_audio", 0).toBool();
	mpSettingsDialog->SetIsRawAudio(mIsRawPrev);
	mpMediaPlayer->SetIsRawAudio(mIsRawPrev);

	mRawFrequencyPrev = pSettings->value("raw_frequency", 8000).toInt();
	mpSettingsDialog->SetRawFrequency(mRawFrequencyPrev);
	mpMediaPlayer->SetRawFrequency(mRawFrequencyPrev);

	mRawNumChannelsPrev = pSettings->value("raw_num_channels", 1).toInt();
	mpSettingsDialog->SetRawNumChannels(mRawNumChannelsPrev);
	mpMediaPlayer->SetRawNumChannels(mRawNumChannelsPrev);

	mRawFormatPrev = pSettings->value("raw_format", FMOD_SOUND_FORMAT_PCM16).toInt();
	mpSettingsDialog->SetRawFormat(mRawFormatPrev);
	mpMediaPlayer->SetRawFormat(static_cast<FMOD_SOUND_FORMAT>(mRawFormatPrev));

	int histogramType = pSettings->value("histogram_type", HistogramPlot::EHistogramTypeLukasAccumulative).toInt();
	mHistogramTypePrev = histogramType;
	mpSettingsDialog->SetHistogramType(histogramType);
	mpHistogramPlot->SetHistogramType(histogramType);

	int rowHeight = pSettings->value("row_height", mpTableView->verticalHeader()->defaultSectionSize()).toInt();
	mpSettingsDialog->SetRowHeight(rowHeight);
	RowHeightChanged(rowHeight);

//	bool stable = pSettings->value("stable_sort", true).toBool();
	bool stable = true;
	mpSettingsDialog->SetStableSortChecked(stable);
	mStableSortCheckedPrev = stable;
	mpProxyModel->SetStableSort(stable);

	bool columnsInSettings = false;
	mpProxyModel->GetColumns()->Clear();
	for (int i=0; i<MlfModelData::EColCOUNT; i++)
	{
		QString key = QString("columns/%1").arg(MlfModelData::Columns::GetColumnId((MlfModelData::EColumn)i));
		if (pSettings->contains(key))
		{
			if (pSettings->value(key).toBool())
			{
				columnsInSettings = true;
				mpProxyModel->GetColumns()->AddColumn((MlfModelData::EColumn)i);
			}
		}
	}
	if (!columnsInSettings)
	{
		mpProxyModel->GetColumns()->AddColumn(MlfModelData::EColFilename);
		mpProxyModel->GetColumns()->AddColumn(MlfModelData::EColStartTime);
		mpProxyModel->GetColumns()->AddColumn(MlfModelData::EColEndTime);
		mpProxyModel->GetColumns()->AddColumn(MlfModelData::EColLabel);
//		mpProxyModel->GetColumns()->AddColumn(MlfModelData::EColScoreRaw);
		mpProxyModel->GetColumns()->AddColumn(MlfModelData::EColScoreLlr);
		mpProxyModel->GetColumns()->AddColumn(MlfModelData::EColScoreNorm);
//		mpProxyModel->GetColumns()->AddColumn(MlfModelData::EColHardDecision);
		mpProxyModel->GetColumns()->AddColumn(MlfModelData::EColCorrect);
	}
	mpProxyModel->ColumnsChanged();
	mpSettingsDialog->SetColumns(mpProxyModel->GetColumns());

	int border = 50;
	QRect default_geom = QApplication::desktop()->screenGeometry();
	default_geom.adjust(border, border, -border, -border);
//	setGeometry(pSettings->value("MainWindow/geometry", default_geom).toRect());
	setGeometry(default_geom);
}

void MainWindow::closeEvent(QCloseEvent *event)
{
//	mpSettings->setValue("MainWindow/geometry", geometry());
	event->accept();
}

void MainWindow::Settings()
{
	mColumnsCountPrev = mpProxyModel->GetColumns()->Size();

	int res = mpSettingsDialog->exec();

	DBG("Settings res = "<<res);
	if (res == QDialog::Accepted)
	{
		QString charset = mpSettingsDialog->GetEncoding();
		if (mEncodingPrev != charset)
		{
			mEncodingPrev = charset;
			mpMlfModel->SetEncoding(charset);
			mpSettings->setValue("encoding", charset);
		}

		MlfModelData::EInputScoreType scoretype = mpSettingsDialog->GetInputScoreType();
		DBG("scoretype: "<<scoretype);
		if (mInputScoreTypePrev != scoretype)
		{
			mInputScoreTypePrev = scoretype;
			mpMlfModel->SetInputScoreType(scoretype);
			mpSettings->setValue("input_score_type", scoretype);
		}

		bool stable = mpSettingsDialog->GetStableSortChecked();
		if (mStableSortCheckedPrev != stable)
		{
			mStableSortCheckedPrev = stable;
			mpProxyModel->SetStableSort(stable);
			mpSettings->setValue("stable_sort", stable);
		}

		unsigned int contextMs = mpSettingsDialog->GetContextMilliseconds();
		if (mContextMsPrev != contextMs)
		{
			mContextMsPrev = contextMs;
			mpMediaPlayer->SetContextMilliseconds(contextMs);
			mpSettings->setValue("context_ms", contextMs);
		}

		unsigned int referenceBoundaryMs = mpSettingsDialog->GetReferenceBoundaryMilliseconds();
		if (mReferenceBoundaryMsPrev != referenceBoundaryMs)
		{
			mReferenceBoundaryMsPrev = referenceBoundaryMs;
			mpMlfModel->SetReferenceBoundaryMilliseconds(referenceBoundaryMs);
//			mpMediaPlayer->SetReferenceBoundaryMilliseconds(contextMs);
		}

		QString	audioFileSuffix = mpSettingsDialog->GetAudioFileSuffix();
		if (mAudioFileSuffixPrev != audioFileSuffix)
		{
			mAudioFileSuffixPrev = audioFileSuffix;
			mpMediaPlayer->SetAudioFileSuffix(audioFileSuffix);
			mpSettings->setValue("audio_file_suffix", audioFileSuffix);
		}

		bool isRaw = mpSettingsDialog->GetIsRawAudio();
		if (mIsRawPrev != isRaw)
		{
			mIsRawPrev = isRaw;
			mpMediaPlayer->SetIsRawAudio(isRaw);
			mpSettings->setValue("is_raw_audio", isRaw);
		}

		int	rawFrequency = mpSettingsDialog->GetRawFrequency();
		if (mRawFrequencyPrev != rawFrequency)
		{
			mRawFrequencyPrev = rawFrequency;
			mpMediaPlayer->SetRawFrequency(rawFrequency);
			mpSettings->setValue("raw_frequency", rawFrequency);
		}

		int	rawNumChannels = mpSettingsDialog->GetRawNumChannels();
		if (mRawNumChannelsPrev != rawNumChannels)
		{
			mRawNumChannelsPrev = rawNumChannels;
			mpMediaPlayer->SetRawNumChannels(rawNumChannels);
			mpSettings->setValue("raw_num_channels", rawNumChannels);
		}

		int	rawFormat = mpSettingsDialog->GetRawFormat();
		if (mRawFormatPrev != rawFormat)
		{
			mRawFormatPrev = rawFormat;
			mpMediaPlayer->SetRawFormat(static_cast<FMOD_SOUND_FORMAT>(rawFormat));
			mpSettings->setValue("raw_format", rawFormat);
		}

		unsigned char histogramType = mpSettingsDialog->GetHistogramType();
		if (mHistogramTypePrev != histogramType)
		{
			mHistogramTypePrev = histogramType;
			mpHistogramPlot->update();
			mpHistogramPlot->SetHistogramType(histogramType);
			mpSettings->setValue("histogram_type", (int)histogramType);
		}
		mpProxyModel->ColumnsChanged();
		mpTableView->resizeColumnsToContents();

//		mpTableView->verticalHeader()->setDefaultSectionSize(10);
//		mpTableView->verticalHeader()->resizeSection(0, 10);
	}
}

void MainWindow::Open()
{
	DBG("MainWindow::Open()");
#ifndef PREDEFINED_PATHS	
	mpOpenDialog->exec();

	QString filenameMlf = mpOpenDialog->GetMlfFilename().trimmed();
	QString filenameRefMlf = mpOpenDialog->GetRefMlfFilename().trimmed();
	QString pathWavs = mpOpenDialog->GetWavsPath().trimmed();
#else
//	QString filename = "/home/miso/projects/mlf_processor/dev_test.mlf";
	#ifdef WIN32
		QString filenameMlf = "c:/projects/mlf_processor/data/dev_train_nonull.mlf";
		QString filenameRefMlf = "c:/projects/mlf_processor/data/ref_work_wavs_MO_filt_mapped_train_lc_win.mlf";
	#else
		QString filenameMlf = "/home/miso/projects/mlf_processor/data/dev_train_nonull.mlf";
		QString filenameRefMlf = "/home/miso/projects/mlf_processor/data/ref_work_wavs_MO_filt_mapped_train_lc_win.mlf";

		//QString filenameMlf = "/home/miso/projects/mlf_processor/data/russian_mlfs/output.det.mlf";
		//QString filenameRefMlf = "/home/miso/projects/mlf_processor/data/russian_mlfs/orig_uc_filt_segm_aligned_grouped_utf.mlf";

		//QString filenameMlf = "/home/miso/projects/mlf_processor/data/russian_cp1251/concat_thr-5.mlf";
		//QString filenameRefMlf = "/home/miso/projects/mlf_processor/data/russian_cp1251/orig_uc_filt_segm_aligned_grouped.mlf";

		//QString filenameMlf = "/home/miso/projects/mlf_processor/data/test_inconsistency.mlf";
		//QString filenameRefMlf = "/home/miso/projects/mlf_processor/data/test_inconsistency_ref.mlf";

		//QString filenameMlf = "/home/miso/projects/mlf_processor/data/spdat.mlf";
		//QString filenameRefMlf = "/home/miso/projects/mlf_processor/data/spdat_ref.mlf";

		//QString filenameMlf = "/home/miso/projects/mlf_processor/data/test_raw.mlf";
		//QString filenameRefMlf = "";

		QString pathWavs = "./wav";
	#endif
#endif
	DBG("mFilenameMlfPrev:"<<mFilenameMlfPrev.toStdString()<<" filenameMlf:"<<filenameMlf.toStdString());
	//if (mFilenameMlfPrev != filenameMlf)
	if (filenameMlf != "")
	{
		OpenMlf(filenameMlf);
		mFilenameMlfPrev = filenameMlf;
	}
	//if (mFilenameRefMlfPrev != filenameRefMlf)
	if (filenameRefMlf != "")
	{
		OpenRefMlf(filenameRefMlf);
		mFilenameRefMlfPrev = filenameRefMlf;
	}
	//if (mPathWavsPrev != pathWavs)
	if (pathWavs != "")
	{
		SelectAudioFilesDirectory(pathWavs);
		mPathWavsPrev = pathWavs;
	}
}

void MainWindow::OpenMlf(QString filename)
{
	if (filename == "")
	{
		return;
	}
	if (!QFile::exists(filename))
	{
		QMessageBox::critical(NULL, tr("MLF Toy"), tr("The file \"").append(filename).append(tr("\" does not exist")), QMessageBox::Ok, QMessageBox::Ok);
		return;
	}
	mpLabelProcessing->setText(tr("Loading MLF..."));
	mpProgressbarProcessing->setValue(0);
	mpFrameProcessing->show();
	mpMlfModel->LoadFromFile(filename);
	DBG("mpProxyModel->SynchronizeMap()...");
	mpProxyModel->SynchronizeMap();
	DBG("mpProxyModel->SynchronizeMap()...done");
//	mpKeywordsFilter->SetMlfModelData(mpMlfModel->GetData());
	mpFrameProcessing->hide();

	mpTableView->setModel(NULL);
	//mpTableView->setModel(mpMlfModel);
	mpTableView->setModel(mpProxyModel);
	mpTableView->resizeColumnsToContents();
	connect(mpTableView->selectionModel(), SIGNAL(selectionChanged ( const QItemSelection & selected, const QItemSelection & deselected )), this, SLOT(TableSelectionChanged ( const QItemSelection & selected, const QItemSelection & deselected )));
//	mpTableView->horizontalHeader()->resizeSection(0, 200);
//	mpTableView->horizontalHeader()->setStretchLastSection(true);
	DBG("MainWindow::OpenMlf()...done");
}

void MainWindow::OpenRefMlf(QString filename)
{
	if (filename == "")
	{
		return;
	}
	if (!QFile::exists(filename))
	{
		QMessageBox::critical(NULL, tr("MLF Toy"), tr("The file \"").append(filename).append(tr("\" does not exist")), QMessageBox::Ok, QMessageBox::Ok);
		return;
	}
	mpLabelProcessing->setText(tr("Loading reference MLF..."));
	mpProgressbarProcessing->setValue(0);
	mpFrameProcessing->show();
	mpMlfModel->LoadRefFromFile(filename);
	mpFrameProcessing->hide();

	mpTableView->setModel(NULL);
	//mpTableView->setModel(mpMlfModel);
	mpTableView->setModel(mpProxyModel);
	mpTableView->resizeColumnsToContents();
//	mpTableView->horizontalHeader()->setStretchLastSection(true);

	connect(mpTableView->selectionModel(), SIGNAL(selectionChanged ( const QItemSelection & selected, const QItemSelection & deselected )), this, SLOT(TableSelectionChanged ( const QItemSelection & selected, const QItemSelection & deselected )));
	mpProxyModel->Resort(7);
}

void MainWindow::SaveMlf()
{
	QString filename = QFileDialog::getSaveFileName(this, tr("Save Mlf"), "", tr("MLF Files (*.mlf)"));
//	QString filename = "/home/miso/projects/mlf_processor/test_saved.mlf";
	if (filename != "")
	{
		mpMlfModel->SaveToFile(filename);
	}
}

void MainWindow::SaveRefMlf()
{
	QString filename = QFileDialog::getSaveFileName(this, tr("Save reference Mlf"), "", tr("MLF Files (*.mlf)"));
//	QString filename = "/home/miso/projects/mlf_processor/test_saved.mlf";
	if (filename != "")
	{
		mpMlfModel->SaveRefToFile(filename);
	}
}

void MainWindow::SelectAudioFilesDirectory(QString dir)
{
	if (dir != "")
	{
		mpMediaPlayer->SetAudioFilesDirectory(dir);
	}
}

void MainWindow::ThresholdSliderChanged(int v)
{
	int mul = pow((float)10,mThresholdPrecision);
	double f = (double)v/mul;
	if (cmp_float_precision(mThresholdValue, f, mThresholdPrecision) != 0)
	{
		mThresholdValue = f;
		mpSpinboxThreshold->setValue(f);
		ThresholdChanged();
	}
}

void MainWindow::ThresholdSpinboxChanged(double v)
{
	if (cmp_float_precision(mThresholdValue, v, mThresholdPrecision) != 0)
	{
		mThresholdValue = v;
		mpSliderThreshold->setValue(to_int(v*pow((float)10,mThresholdPrecision)));
		ThresholdChanged();
	}
}

void MainWindow::ThresholdChanged()
{
	mpMlfModel->ThresholdChanged(mpSpinboxThreshold->value());
	mpTableItemDelegate->ThresholdChanged(mpSpinboxThreshold->value());
#ifdef SHOW_HD1_ONLY
	if (mpCheckboxShowOnlyHD1->isChecked())
	{
		// because of filtering out the HD=0 detections which depend on the threshold
		mpProxyModel->FilterChanged(); 
	}
#endif
}

void MainWindow::PriorSliderChanged(int v)
{
	int mul = pow((float)10,mPriorPrecision);
	double f = (double)v/mul;
	if (cmp_float_precision(mPriorValue, f, mPriorPrecision) != 0)
	{
		mPriorValue = f;
		mpSpinboxPrior->setValue(f);
		PriorChanged();
	}
}

void MainWindow::PriorSpinboxChanged(double v)
{
	if (cmp_float_precision(mPriorValue, v, mPriorPrecision) != 0)
	{
		mPriorValue = v;
		mpSliderPrior->setValue(to_int(v*pow((float)10,mPriorPrecision)));
		PriorChanged();
	}
}

void MainWindow::PriorChanged()
{
	mpMlfModel->PriorChanged(mpSpinboxPrior->value());
#ifdef SHOW_HD1_ONLY
	if (mpCheckboxShowOnlyHD1->isChecked())
	{
		// because of filtering out the HD=0 detections which depend on the threshold
		mpProxyModel->FilterChanged(); 
	}
#endif
}

void MainWindow::RadiobuttonLLRToggled(bool checked)
{
	if (checked)
	{
		mpMlfModel->SetScoreTypeLLR();
	}
}

void MainWindow::RadiobuttonKSToggled(bool checked)
{
	if (checked)
	{
		mpMlfModel->SetScoreTypeKS();
		mpGroupboxMultiplicative->show();
		mpGroupboxAdditive->show();
#ifdef SHOW_THRESHOLD
		set_slider_spinbox_pair(mpSliderThreshold, mpSpinboxThreshold, 0, 1, mThresholdPrecision, 0.5, &mThresholdValue);
#endif
		set_slider_spinbox_pair(mpSliderMultiplicative, mpSpinboxMultiplicative, 0, 5, mMultiplicativePrecision, mMultiplicativeDefaultValue, &mMultiplicativeValue);
	}
	else
	{
		mpGroupboxMultiplicative->hide();
		mpGroupboxAdditive->hide();
#ifdef SHOW_THRESHOLD
		set_slider_spinbox_pair(mpSliderThreshold, mpSpinboxThreshold, mThresholdMin, mThresholdMax, mThresholdPrecision, mThresholdDefaultValue, &mThresholdValue);
#endif
		set_slider_spinbox_pair(mpSliderMultiplicative, mpSpinboxMultiplicative, mMultiplicativeMin, mMultiplicativeMax, mMultiplicativePrecision, mMultiplicativeDefaultValue, &mMultiplicativeValue);
	}
}

void MainWindow::MultiplicativeSliderChanged(int v)
{
	DBG("MainWindow::MultiplicativeSliderChanged()");
	int mul = pow((float)10,mMultiplicativePrecision);
	double f = (double)v/mul;
	DBG("mul="<<mul<<" f="<<f);
	if (cmp_float_precision(mMultiplicativeValue, f, mMultiplicativePrecision) != 0 && mMultiplicativeValue <= mMultiplicativeMax && mMultiplicativeValue >= mMultiplicativeMin)
	{
		DBG("mMultiplicativeValue != f ..."<<mMultiplicativeValue<<" != "<<f);
		mMultiplicativeValue = f;
		DBG("f:"<<f<<" mul:"<<mul<<" mMultiplicativeValue_new:"<<mMultiplicativeValue);
		mpSpinboxMultiplicative->setValue(f);
		MultiplicativeChanged();
	}
}

void MainWindow::MultiplicativeSpinboxChanged(double v)
{
	DBG("MainWindow::MultiplicativeSpinboxChanged()");
	if (cmp_float_precision(mMultiplicativeValue, v, mMultiplicativePrecision) != 0 && mMultiplicativeValue <= mMultiplicativeMax && mMultiplicativeValue >= mMultiplicativeMin)
	{
		DBG("mMultiplicativeValue != v ..."<<mMultiplicativeValue<<" != "<<v);
		mMultiplicativeValue = v;
		DBG("mpSliderMultiplicative->setValue("<<to_int(v*pow((float)10,mMultiplicativePrecision))<<") ...(int)"<<(v*pow((float)10,mMultiplicativePrecision)));
		mpSliderMultiplicative->setValue(to_int(v*pow((float)10,mMultiplicativePrecision)));
		MultiplicativeChanged();
	}
}

void MainWindow::MultiplicativeChanged()
{
	DBG("MainWindow::MultiplicativeChanged()");
	mpMlfModel->MultiplicativeChanged(mpSpinboxMultiplicative->value());
#ifdef SHOW_HD1_ONLY
	if (mpCheckboxShowOnlyHD1->isChecked())
	{
		// because of filtering out the HD=0 detections which depend on the threshold
		mpProxyModel->FilterChanged(); 
	}
#endif
}

void MainWindow::AdditiveSliderChanged(int v)
{
	int mul = pow((float)10,mAdditivePrecision);
	double f = (double)v/mul;
	DBG("mul="<<mul<<" f="<<f);
	if (cmp_float_precision(mAdditiveValue, f, mAdditivePrecision) != 0 && mAdditiveValue <= mAdditiveMax && mAdditiveValue >= mAdditiveMin)
	{
		mAdditiveValue = f;
		DBG("mpSpinboxAdditive->setValue("<<f<<")");
		mpSpinboxAdditive->setValue(f);
		AdditiveChanged();
	}
}

void MainWindow::AdditiveSpinboxChanged(double v)
{
	if (cmp_float_precision(mAdditiveValue, v, mAdditivePrecision) != 0 && mAdditiveValue <= mAdditiveMax && mAdditiveValue >= mAdditiveMin)
	{
		mAdditiveValue = v;
		DBG("mpSliderAdditive->setValue("<<to_int(v*pow((float)10,mAdditivePrecision))<<")");
		mpSliderAdditive->setValue(to_int(v*pow((float)10,mAdditivePrecision)));
		AdditiveChanged();
	}
}

void MainWindow::AdditiveChanged()
{
	DBG("AdditiveChanged(): "<<mpSpinboxAdditive->value());
	mpMlfModel->AdditiveChanged(mpSpinboxAdditive->value());
#ifdef SHOW_HD1_ONLY
	if (mpCheckboxShowOnlyHD1->isChecked())
	{
		// because of filtering out the HD=0 detections which depend on the threshold
		mpProxyModel->FilterChanged(); 
	}
#endif
}

void MainWindow::MlfScannerProgress(int percent)
{
	mpProgressbarProcessing->setValue(percent);
//	repaint();
}

void MainWindow::SortProgress(int percent)
{
	mpFrameProcessing->show();
	mpProgressbarProcessing->setValue(percent);
}

void MainWindow::SortProgressStart()
{
	mpLabelProcessing->setText(tr("Sorting..."));
	mpProgressbarProcessing->setValue(0);
	mpProgressbarProcessing->show();
	mpFrameProcessing->show();
	mpTimerHideStatusbar->stop();
}

void MainWindow::SortProgressEnd()
{
	mpLabelProcessing->setText(tr("Sorting...done"));
	mpProgressbarProcessing->hide();
	mpTimerHideStatusbar->start(1000);
}

void MainWindow::HideStatusbar()
{
	mpFrameProcessing->hide();
}

void MainWindow::TableItemDoubleClicked(const QModelIndex & index)
{
	Q_UNUSED(index);
//	DBG("MainWindow::TableItemDoubleClicked("<<index.row()<<", "<<index.column()<<")");

//	mpMediaPlayer->raise();

	if ((*mpProxyModel->GetColumns())[index.column()] == MlfModelData::EColCorrect)
	{
		mpProxyModel->ChangeCorrect(index.row());
	}
	else
	{
		mpMediaPlayer->PlayDetection();
	}
}

void MainWindow::TableItemClicked(const QModelIndex & index)
{
	mpMediaPlayer->setEnabled(true);

	MlfModelData::MlfTableRow* row = mpProxyModel->GetRow(index.row());
	DBG("selected row:"<<*row);

	mpSelectedRow = row;
	*mpSelectedIndex = index;
	SetCurrentRow(mpSelectedRow);
}

void MainWindow::SetCurrentRow(MlfModelData::MlfTableRow * row)
{
//	const unsigned int boundary_time_ms = mpSettingsDialog->GetContextMilliseconds();
	quint64 seek_ms = row->mpMlfRecord->mStartTime;

	qint64 stop_ms = row->mpMlfRecord->mEndTime;
//	qint64 stop_ms = row->mpMlfRecord->mEndTime + boundary_time_ms;
	DBG("seek_ms:"<<seek_ms<<" stop_ms:"<<stop_ms);

	mpMediaPlayer->SetFilename(row->mFilename.c_str());
	mpMediaPlayer->SetLabPath(row->mLabPath);
	mpMediaPlayer->SetDetectionStartMs(seek_ms);
	mpMediaPlayer->SetDetectionEndMs(stop_ms);
}

void MainWindow::ShowKeywordsFilter()
{
	mpKeywordsFilter->show();
}

void MainWindow::FilterChanged()
{
	mpProxyModel->FilterChanged();
	DBG("MainWindow::FilterChanged() ..."<<mpTableView->verticalHeader()->count());
//	mpTableView->verticalHeader()->sectionCountChanged(old_count, mpProxyModel->rowCount());
//	mpTableView->verticalHeader()->headerDataChanged(Qt::Vertical, 0, 5);
}

void MainWindow::OnHistogramStatisticsChanged()
{
	mpLabelHistogramHits->setText(tr("Hits (%1)").arg(mpHistogramPlot->GetCountHits()));
	mpLabelHistogramMisses->setText(tr("Misses (%1)").arg(mpHistogramPlot->GetCountMisses()));
	mpLabelHistogramFalseAlarms->setText(tr("False alarms (%1)").arg(mpHistogramPlot->GetCountFalseAlarms()));
/*
	mpLabelHistogramHits->setText(tr("Hits (%1)").arg(mpHistogramPlot->GetCountHits(HistogramPlot::EHistogramTypeIndexIgiAccumulative)));
	mpLabelHistogramMisses->setText(tr("Misses (%1)").arg(mpHistogramPlot->GetCountMisses(HistogramPlot::EHistogramTypeIndexIgiAccumulative)));
	mpLabelHistogramFalseAlarms->setText(tr("False alarms (%1)").arg(mpHistogramPlot->GetCountFalseAlarms(HistogramPlot::EHistogramTypeIndexIgiAccumulative)));
*/
}

void MainWindow::OnDataChanged()
{
	mpTableView->selectionModel()->clearSelection();
	OnHistogramStatisticsChanged();
//	mpTableView->verticalHeader()->
//	mpTableView->viewport()->update();
}

void MainWindow::CheckboxShowOnlyHD1Toggled(bool checked)
{
	mpProxyModel->SetShowOnlyHD1(checked);
	mpProxyModel->FilterChanged();
}

void MainWindow::CopyToClipboard()
{
	DBG("MainWindow::CopyToClipboard()");
	if (mpSelectedIndex)
	{
		mpClipboard->setText(mpProxyModel->data(*mpSelectedIndex).toString());
	}
}

void MainWindow::TableContextMenuRequested(const QPoint & pos)
{
	DBG("TableContextMenuRequested");
	if (mpTableView->contentsRect().contains(pos))
	{
		DBG("contained");
		*mpSelectedIndex = mpTableView->indexAt(pos);
		DBG("row:"<<mpSelectedIndex->row()<<" col:"<<mpSelectedIndex->column());
//		DBG("row:"<<mpTableView->rowAt(pos.y)<<" col:"<<mpTableView->columnAt(pos.y));
		QMenu menu(this);
		menu.addAction(mpActionCopyToClipboard);
		menu.exec(mpTableView->mapToGlobal(pos));
	//	menu.exec(pos);
	}
}

void MainWindow::TableSelectionChanged ( const QItemSelection & selected, const QItemSelection & deselected )
{
	Q_UNUSED(deselected);
	if (!selected.indexes().empty())
	{
		mpMediaPlayer->show();

		QModelIndex index = selected.indexes().first();

		MlfModelData::MlfTableRow* row = mpProxyModel->GetRow(index.row());
		DBG("selected row:"<<*row);

		mpSelectedRow = row;
		*mpSelectedIndex = index;
		SetCurrentRow(mpSelectedRow);
	}
}

void MainWindow::RowHeightChanged(int height)
{
	for (int i=0; i<mpTableView->verticalHeader()->count(); i++)
	{
		mpTableView->verticalHeader()->resizeSection(i, height);
	}
}

void MainWindow::FrameControlsResized(QResizeEvent * event)
{
//	DBG("MainSplitter::resizeEvent() oldSize:"<<event->oldSize().width()<<" size:"<<event->size().width());
	int w1 = event->oldSize().width();
	int w2 = event->size().width();
	if (w2 > w1)
	{
		mpMainSplitter->setSizes(mpMainSplitter->sizes());
	}
}

