#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QWidget>
#include <QMainWindow>
#include <QMap>
#include <QFrame>
#include <QAbstractTableModel>
#include "mlfmodel.h"
#include "mlfsortfilterproxymodel.h"
#include "tableitemdelegate.h"
#include "dbg.h"
#include "mlf_scanner.h"
#include "mlfsortingthread.h"
#include "mediaplayer.h"
#include "histogramplot.h"
#include "keywordsfilter.h"
#include "opendialog.h"
#include "settingsdialog.h"
#include "aboutwindow.h"
#include "tutorialwindow.h"
//#include "common.h"

class QLabel;
class QPushButton;
class QTableView;
class QSlider;
class QDoubleSpinBox;
class QRadioButton;
class QGroupBox;
class QSortFilterProxyModel;
class QProgressBar;
class QStatusBar;
class QFrame;
class QTimer;
class QCheckBox;
class QSplitter;
class QSettings;

class FrameControlsScore : public QFrame
{
	Q_OBJECT
	public:
		FrameControlsScore(QWidget * parent = 0, Qt::WindowFlags f = 0) : QFrame(parent, f) {}
	signals:
		void OnResize(QResizeEvent * event);
	protected:
		void resizeEvent(QResizeEvent * event)
		{
			emit OnResize(event);
		}
};

class MainWindow : public QMainWindow, public IMlfScannerProgressDisplay, public ISortProgressDisplay
{
	Q_OBJECT

	public:
		MainWindow(QSettings * pSettings, QWidget *parent = 0);
		void MlfScannerProgress(int percent);
		void SortProgress(int percent);
		void SortProgressStart();
		void SortProgressEnd();
		void SetCurrentRow(MlfModelData::MlfTableRow * row);
		void ReadSettings(QSettings * pSettings);

	public slots:
		void Open();
		void Settings();
		void OpenMlf(QString filename);
		void OpenRefMlf(QString filename);
		void SelectAudioFilesDirectory(QString dir);
		void SaveMlf();
		void SaveRefMlf();
		void ThresholdSliderChanged(int v);
		void ThresholdSpinboxChanged(double v);
		void ThresholdChanged();
		void PriorSliderChanged(int v);
		void PriorSpinboxChanged(double v);
		void PriorChanged();
		void MultiplicativeSliderChanged(int v);
		void MultiplicativeSpinboxChanged(double v);
		void MultiplicativeChanged();
		void AdditiveSliderChanged(int v);
		void AdditiveSpinboxChanged(double v);
		void AdditiveChanged();
		void RadiobuttonLLRToggled(bool checked);
		void RadiobuttonKSToggled(bool checked);
		void HideStatusbar();
		void TableItemDoubleClicked(const QModelIndex & index);
		void TableItemClicked(const QModelIndex & index);
		void ShowKeywordsFilter();
		void FilterChanged();
		void OnDataChanged();
		void OnHistogramStatisticsChanged();
		void CheckboxShowOnlyHD1Toggled(bool checked);
		void CopyToClipboard();
		void TableContextMenuRequested(const QPoint & pos);
		void TableSelectionChanged ( const QItemSelection & selected, const QItemSelection & deselected );
		void RowHeightChanged(int height);
		void FrameControlsResized(QResizeEvent * event);

	protected:
		void closeEvent(QCloseEvent *event);

	private:
		QTableView *				mpTableView;
		MlfModel *					mpMlfModel;
		//QSortFilterProxyModel *	mpProxyModel;
		MlfSortFilterProxyModel *	mpProxyModel;
		TableItemDelegate *			mpTableItemDelegate;
		HistogramPlot *				mpHistogramPlot; 
		MlfModelData::MlfTableRow * mpSelectedRow;

		MediaPlayer *				mpMediaPlayer;

		KeywordsFilter * 	mpKeywordsFilter;

		OpenDialog *		mpOpenDialog;

		SettingsDialog *	mpSettingsDialog;

		TutorialWindow *    mpTutorialWindow;

		AboutWindow *       mpAboutWindow;

		QPushButton *		mpButtonOpenMlf;
		QPushButton *		mpButtonOpenRefMlf;
		QPushButton *		mpButtonSaveMlf;
		QPushButton *		mpButtonSaveRefMlf;
		QPushButton * 		mpButtonSelectAudioFilesDirectory;
		QPushButton * 		mpButtonKeywordsFilter;
		QPushButton *		mpButtonQuit;
		QSlider *			mpSliderThreshold;
		QDoubleSpinBox *	mpSpinboxThreshold;
		QSlider *			mpSliderPrior;
		QDoubleSpinBox *	mpSpinboxPrior;
		QRadioButton *		mpRadiobuttonLLR;
		QRadioButton *		mpRadiobuttonKS;
		QSlider *			mpSliderMultiplicative;
		QDoubleSpinBox *	mpSpinboxMultiplicative;
		QGroupBox *			mpGroupboxMultiplicative;
		QSlider *			mpSliderAdditive;
		QDoubleSpinBox *	mpSpinboxAdditive;
		QGroupBox *			mpGroupboxAdditive;
		QLabel *			mpLabelProcessing;
		QProgressBar *		mpProgressbarProcessing;
		QGroupBox *			mpGroupboxProcessing;
		QStatusBar *		mpStatusBar;
		QFrame *			mpFrameProcessing;
		QTimer *			mpTimerHideStatusbar;
		QCheckBox * 		mpCheckboxShowOnlyHD1;
		QLabel * 			mpLabelHistogramHits;
		QLabel * 			mpLabelHistogramMisses;
		QLabel * 			mpLabelHistogramFalseAlarms;
		QSplitter *			mpMainSplitter;

		QAction * 			mpActionCopyToClipboard;
		QClipboard *		mpClipboard;
		QModelIndex	*		mpSelectedIndex;

		QSettings * 		mpSettings;

		QString 			mFilenameMlfPrev;
		QString 			mFilenameRefMlfPrev;
		QString 			mPathWavsPrev;
		QString				mEncodingPrev;
		MlfModelData::EInputScoreType mInputScoreTypePrev;
		bool 				mStableSortCheckedPrev;
		unsigned int		mContextMsPrev;
		unsigned int 		mColumnsCountPrev;
		unsigned int 		mReferenceBoundaryMsPrev;
		unsigned char		mHistogramTypePrev;
		QString				mAudioFileSuffixPrev;
		bool				mIsRawPrev;
		int					mRawFrequencyPrev;
		int					mRawNumChannelsPrev;
		int					mRawFormatPrev;
/*
		bool				mColumnFilenamePrev;
		bool				mColumnStartTimePrev;
		bool				mColumnEndTimePrev;
		bool				mColumnLabelPrev;
		bool				mColumnScoreRawPrev;
		bool				mColumnScoreLlrPrev;
		bool				mColumnScoreNormPrev;
		bool				mColumnHardDecisionPrev;
		bool				mColumnCorrectPrev;
*/
		double mThresholdValue;
		double mPriorValue;
		double mMultiplicativeValue;
		double mAdditiveValue;

		static const int mThresholdMin = -100;
		static const int mThresholdMax = 100;
		static const int mThresholdPrecision = 2;
		static const int mThresholdDefaultValue = 0;

		static const int mPriorMin = -10;
		static const int mPriorMax = 10;
		static const int mPriorPrecision = 2;
		static const int mPriorDefaultValue = 0;

		static const int mMultiplicativeMin = -10;
		static const int mMultiplicativeMax = 10;
		static const int mMultiplicativePrecision = 2;
		static const float mMultiplicativeDefaultValue = 1;

		static const int mAdditiveMin = -10;
		static const int mAdditiveMax = 10;
		static const int mAdditivePrecision = 2;
		static const int mAdditiveDefaultValue = 0;

		int CmpFloatPrecision(double a, double b, int precision);
};


#endif

