#ifndef MEDIAPLAYERTHREAD_H
#define MEDIAPLAYERTHREAD_H

#include <QMutex>
#include <QSize>
#include <QThread>
#include <QWaitCondition>

#include "fmod.h"
#include "fmod_errors.h"
#include "dbg.h"

class MediaPlayerThread : public QThread
{
		Q_OBJECT
	public:
		enum EState { StatePlaying, StatePaused, StateStopped };

	    MediaPlayerThread(QObject *parent = 0);
	    ~MediaPlayerThread();

		void SetAudioFilesDirectory(const QString& dir) { mAudioFilesDirectory = dir; }
		void SetFilename(const QString& filename) { mFilename = filename; }
		void SetLabPath(const QString& path) { mLabPath = path; }
		void SetDetectionStartMs(qint64 seekMs);
		void SetDetectionEndMs(qint64 stopMs);
		void SetUpdateIntervalMs(unsigned int ms) { mUpdateIntervalMs = ms; }
		void SetContextMilliseconds(unsigned int ms) { mContextMs = ms; }
		void SetAudioFileSuffix(QString suffix) { mAudioFileSuffix = suffix; }
		void SetIsRawAudio(bool isRaw) { mIsRaw = isRaw; }
		void SetRawFrequency(int freq) { mExinfo.defaultfrequency = freq; }
		void SetRawNumChannels(int n) { mExinfo.numchannels = n; }
		void SetRawFormat(FMOD_SOUND_FORMAT fmt) { mExinfo.format = fmt; }

		EState GetState() { return mState; }

		void Play();
		void PlayDetection();
		void Pause();
		void Stop();

	public slots:
		void SeekTo(unsigned int posMs);

	signals:
		void PlayingStarted();
		void Paused();
		void Stopped();
		void AudioFileLengthChanged(unsigned int lengthMs);
		void AudioFilePositionChanged(unsigned int posMs);
		void Error(const QString& msg);

	protected:
		void run();

		QMutex mMutex;
		QWaitCondition mCondition;

		bool mRestart;
		bool mAbort;

		FMOD_SYSTEM *		mpFmodSystem;
		FMOD_SOUND *		mpFmodSound;
		FMOD_CHANNEL *		mpFmodChannel;
		FMOD_CREATESOUNDEXINFO mExinfo;
		unsigned int      	mFmodVersion;

		unsigned int		mContextMs;
		qint64				mSeekMs;
		bool				mSeek;
		unsigned int		mAudioLengthMs;
		qint64				mDetectionStartMs;
		qint64				mDetectionEndMs;
		QString				mFilename;
		QString				mLabPath;
		QString				mLoadedFilename;
		QString				mAudioFilesDirectory;
		unsigned int		mUpdateIntervalMs;
		EState				mState;
		unsigned int		mPositionMs;
		unsigned int		mLengthMs;
		unsigned int		mLengthMsPrevious;
		bool				mPlayDetection;
		bool                mIsRaw;
		QString				mAudioFileSuffix;

//		void ERRCHECK(FMOD_RESULT result);
		void SetState(EState state);
		bool LoadAudioFile();
};

#endif

