#ifndef MLF_H
#define MLF_H

#include <string>
#include <list>
#include <map>

struct MlfRecord
{
	public:
		unsigned long mStartTime;
		unsigned long mEndTime;
		std::string mLabel;
		double mScore1;
		double mScore2;
		int mHardDecision;

		MlfRecord() : mStartTime(0), mEndTime(0), mLabel(""), mScore1(0.0), mScore2(0.0), mHardDecision(0) {}

		friend std::ostream& operator<<(std::ostream& os, const MlfRecord& rec)
		{
			return os << rec.mStartTime << " " << rec.mEndTime << " " << rec.mLabel << " " << rec.mScore1 << " " << rec.mScore2 << " " << rec.mHardDecision;
		}

		void Init() 
		{
			mStartTime=0;
			mEndTime=0;
			mLabel="";
			mScore1=0.0;
			mScore2=0.0;
			mHardDecision=0;
		}
};

class MlfOneFile : public std::list<MlfRecord*>
{
	protected:
		std::string mFilename;
	public:
/*		~MlfOneFile()
		{
			for (iterator i=begin(); i!=end(); i++)
			{
				delete *i;
			}
		}
*/
		friend std::ostream& operator<<(std::ostream& os, const MlfOneFile& f)
		{
			os << f.mFilename << std::endl;
			for (const_iterator i=f.begin(); i!=f.end(); i++)
			{
				os << **i << std::endl;
			}
			os << "." << std::endl;
			return os;
		}

		void Add(MlfRecord* pMlfRecord) { push_back(pMlfRecord); }
		void SetFilename(const std::string& filename) { mFilename=filename; }
		const std::string& GetFilename() const { return mFilename; }
};

class Mlf : public std::map<std::string, MlfOneFile*>
{
	private:
		unsigned int mFilesCount;
		unsigned int mRecordsCount;

	public:
		~Mlf()
		{
			Clear();
		}
		
		void Init()
		{
			Clear();
		}

		void Clear()
		{
/*			for (iterator i=begin(); i!=end(); i++)
			{
				delete i->second;
			}
*/			clear();
		}

		void Add(MlfOneFile* pMlfOneFile)
		{
			insert(make_pair(pMlfOneFile->GetFilename(), pMlfOneFile));
		}

		friend std::ostream& operator<<(std::ostream& os, const Mlf& m)
		{
			for (const_iterator i=m.begin(); i!=m.end(); i++)
			{
				os << *(i->second) << std::endl;
			}
			return os;
		}

		unsigned int GetFilesCount() { return mFilesCount; }
		void SetFilesCount(unsigned int count) { mFilesCount = count; }

		unsigned int GetRecordsCount() { return mRecordsCount; }
		void SetRecordsCount(unsigned int count) { mRecordsCount = count; }
};

#endif
