#include <iostream>
#include <sstream>
#include <fstream>
#include <iomanip>
#include <string>
#include <cstdlib>
#include "mlf.h"
#include "dbg.h"

#include "mlf_scanner.h"

#define DBG_TOK(msg) DBG(msg)

#define STATE_PATH 1
#define STATE_RECORD 2

using namespace std;

void MlfScanner::ParseFile(const std::string& filename, Mlf* pMlf)
{
	ifstream in_mlf(filename.c_str());
	in_mlf.seekg (0, ios::end);
	mFilesize = in_mlf.tellg();
	in_mlf.seekg (0, ios::beg);
	if (!in_mlf.good())
	{
		CERR("!in_mlf.good()");
		EXIT();
	}
//	std::cout << "mFilesize: "<<mFilesize<<std::endl;
	mPercent = 0;

	unsigned int files_count = 0;
	unsigned int records_count = 0;
	pMlf->Init();

//	mpProgressDisplay = 0;
	MlfOneFile* mlf_one_file = NULL;

	mLineNumber = 1;

	int buf_size = 1024;
	char buf[buf_size];
	int state = STATE_PATH;
	unsigned int pos = 0;
	while (in_mlf.getline(buf, buf_size))
	{
		//DBG("buf:"<<buf);
		string line = buf;
		pos += line.length();
		if (line[0] == '#')
		{
			// comment
		}
		else
		{
			switch (state)
			{
				case STATE_PATH: 
					if (line[0] == '"')
					{
						mlf_one_file = new MlfOneFile;
						mlf_one_file->SetFilename(line);
						pMlf->Add(mlf_one_file);
						files_count++;
						
						unsigned int percent = (unsigned int)((float)pos/(float)mFilesize*mPercentMax);
						//DBG("percent:"<<percent);
						if (mpProgressDisplay && percent > mPercent) 
						{
							mpProgressDisplay->MlfScannerProgress((int)percent);
							mPercent = percent;
						}
						state = STATE_RECORD;
					}
					break;

				case STATE_RECORD:
					if (line[0] == '.')
					{
						state = STATE_PATH;
					}
					else
					{
//						istringstream ss(line);
						MlfRecord * rec = new MlfRecord;
						vector<string> tokens;
						Tokenize(line, tokens, " \t");

						for (unsigned int i=0; i<tokens.size(); i++)
						{
							string s = tokens[i];
							//DBG("tokens["<<i<<"]: "<<s);
							switch (i)
							{
								case 0: 
									if (s.length() < 4) 
									{ 
										rec->mStartTime = 0;
									}
									else
									{
										s.erase(s.length()-4); // precision up to milliseconds
										//DBG("s: "<<s);
										rec->mStartTime=atol(s.c_str()); 
									}
									break;

								case 1:
									if (s.length() < 4) 
									{ 
										rec->mStartTime = 0;
									}
									else
									{
										s.erase(s.length()-4); // precision up to milliseconds
										rec->mEndTime=atol(s.c_str());
									}
									break;

								case 2:
									rec->mLabel = s;
									break;

								case 3:
									rec->mScore1 = atof(s.c_str());
									break;

								case 4:
									rec->mScore2 = atof(s.c_str());
									break;

								case 5:
									rec->mHardDecision = atol(s.c_str());
									break;
							}
						}
						mlf_one_file->Add(rec);
						records_count++;
					}
			}
		}
	}

	pMlf->SetFilesCount(files_count);
	pMlf->SetRecordsCount(records_count);
}

