#ifndef MLF_SCANNER_H
#define MLF_SCANNER_H

#include <string>
#include <vector>
#include "mlf.h"


class IMlfScannerProgressDisplay {
	public:
		virtual void MlfScannerProgress(int percent) = 0;
};


class MlfScanner {
	public:
		MlfRecord mRec;
		std::string mFilename;
		unsigned int mLineNumber;

		MlfScanner() : mpProgressDisplay(NULL), mPercentMax(100) {}

		void ParseFile(const std::string& filename, Mlf* pMlf);
		void SetMlfScannerProcessingDisplay(IMlfScannerProgressDisplay* pDisplay) { mpProgressDisplay = pDisplay; }
		void SetProgressMaximum(unsigned int max) { mPercentMax = max; }
	private:
		IMlfScannerProgressDisplay* mpProgressDisplay;
		unsigned int mPercentMax;
		unsigned int mPercent;
		unsigned int mFilesize;

		int l;
		char* s;

		void Tokenize(const std::string& str,
					  std::vector<std::string>& tokens,
					  const std::string& delimiters = " ")
		{
			// Skip delimiters at beginning.
			std::string::size_type lastPos = str.find_first_not_of(delimiters, 0);
			// Find first "non-delimiter".
			std::string::size_type pos     = str.find_first_of(delimiters, lastPos);

			while (std::string::npos != pos || std::string::npos != lastPos)
			{
				// Found a token, add it to the vector.
				tokens.push_back(str.substr(lastPos, pos - lastPos));
				// Skip delimiters.  Note the "not_of"
				lastPos = str.find_first_not_of(delimiters, pos);
				// Find next "non-delimiter"
				pos = str.find_first_of(delimiters, lastPos);
			}
		}

};

#endif

