#ifndef MLFTABLEMODEL_H
#define MLFTABLEMODEL_H

#include <QAbstractItemModel>
#include <QVector>
#include <QVariant>
#include <QFileIconProvider>
#include <string>
#include "mlf.h"
#include "mlf_scanner.h"
#include "mlfmodeldata.h"
#include "computescorethread.h"
#include "histogramplot.h"
#include "dbg.h"
#include "mlfsortingthread.h"
//#include "../normalization/mymath.h"
//#include "../normalization.h"
//#include "treeitem.h"

class QFrame;


class MlfModel : public QAbstractTableModel
{
	Q_OBJECT

	private:

	public:
		MlfModel(QObject *parent = 0);
		~MlfModel();

		typedef QMap<QString, int> MlfLabels;

		int rowCount(const QModelIndex &parent = QModelIndex()) const;
		int columnCount(const QModelIndex &parent = QModelIndex()) const;

		QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;
		QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const;
//		void sort ( int column, Qt::SortOrder order = Qt::AscendingOrder );
		int SetItem( int row, const MlfModelData::MlfTableRow& data );
		MlfModelData::MlfTableRow* Row(int r) { return (*mpData)[r]; }

		void LoadFromFile(const QString& filename);
		void LoadRefFromFile(const QString& filename);
		void SaveMlfToFile(const Mlf& mlf, const QString& filename);
		void SaveToFile(const QString& filename);
		void SaveRefToFile(const QString& filename);
		void ThresholdChanged(double t);
		void PriorChanged(double p);
		void MultiplicativeChanged(double m);
		void AdditiveChanged(double a);
		void RecomputeNormalizedScore();
		void SetScoreTypeLLR() { mScoreType = MlfModelData::LogLikelihoodRatio; PriorChanged(mPrior); }
		void SetScoreTypeKS() { mScoreType = MlfModelData::KarlovcoScore; PriorChanged(mPrior); }
		void SetEncoding(QString charsetName);
		void SetInputScoreType(MlfModelData::EInputScoreType inputScoreType);
		void SetHistogramFrame(QFrame* frame) { mpHistogramFrame = frame; }
		void SetSelectedKeywords(MlfSortingThread::SelectedKeywordsMap* pMap) { mpSelectedKeywords = pMap; mComputeScoreThread.SetSelectedKeywords(pMap); }
		void SetProgressDisplay(IMlfScannerProgressDisplay* pProgressDisplay) { mpProgressDisplay = pProgressDisplay; }

		double GetNormalizedScoreMin() { return mNormalizedScoreMin; }
		double GetNormalizedScoreMax() { return mNormalizedScoreMax; }
		double GetThreshold() { return mThreshold; }
		MlfModelData::Data* GetData() { return mpData; }
		HistogramPlot* GetHistogramPlot() { return mComputeScoreThread.GetHistogramPlot(); }
		void SetHistogramPlot(HistogramPlot* plot) { mComputeScoreThread.SetHistogramPlot(plot); }
		void ChangeCorrect(int row);
		void SetReferenceBoundaryMilliseconds(unsigned int ms) { mReferenceBoundaryMs = ms; RefreshReference(); }
		MlfLabels * GetMlfLabels() { return &mMlfLabels; }
		void RefreshHistogram();

	private slots:
		void NormalizedScoreRecomputed(double min, double max);
		void RefreshReference();

	private:
//		QVector<QVariant> mHeaderData;
		Mlf mMlf;
		Mlf mRefMlf;
		unsigned int mReferenceBoundaryMs;
		QString mCharsetName;
		MlfModelData::EInputScoreType mInputScoreType;
		MlfModelData::EScoreType mScoreType;
		double mThreshold;
		double mPrior;
		double mMultiplicative;
		double mAdditive;
		ComputeScoreThread mComputeScoreThread;
		IMlfScannerProgressDisplay * mpProgressDisplay;
//		Normalization mNormalization;

		MlfLabels mMlfLabels;
		MlfModelData::Data* mpData;
		QFrame* mpHistogramFrame;
		MlfSortingThread::SelectedKeywordsMap * mpSelectedKeywords;

		double mNormalizedScoreMin;
		double mNormalizedScoreMax;

};

#endif
