#include "mlfsortfilterproxymodel.h"
#include <iostream>
using namespace std;


MlfSortFilterProxyModel::MlfSortFilterProxyModel (QSettings * pSettings, QObject * parent)
   	: QAbstractProxyModel(parent), mpSettings(pSettings), mColumn(0), mpProgressDisplay(NULL)
{
	mColumns.SetSettings(mpSettings);
/*	mColumns.AddColumn(MlfModelData::EColFilename);
	mColumns.AddColumn(MlfModelData::EColStartTime);
	mColumns.AddColumn(MlfModelData::EColEndTime);
	mColumns.AddColumn(MlfModelData::EColLabel);
	mColumns.AddColumn(MlfModelData::EColScoreRaw);
	mColumns.AddColumn(MlfModelData::EColScorePosterior);
	mColumns.AddColumn(MlfModelData::EColScoreLlr);
	mColumns.AddColumn(MlfModelData::EColScoreNorm);
	mColumns.AddColumn(MlfModelData::EColHardDecision);
	mColumns.AddColumn(MlfModelData::EColCorrect);
*/	mColumnsCountPrev = mColumns.Size();
	mSortingThread.SetColumns(&mColumns);

	connect(&mSortingThread, SIGNAL(SortProgress(int)), this, SLOT(SortProgress(int)));
	connect(&mSortingThread, SIGNAL(SortProgressStart()), this, SLOT(SortProgressStart()));
}

int MlfSortFilterProxyModel::columnCount ( const QModelIndex & parent ) const
{ 
	Q_UNUSED(parent);
	//			DBG("MlfSortFilterProxyModel::columnCount()");
	//return sourceModel()->columnCount(parent); 
	return mColumns.Size();
}
int MlfSortFilterProxyModel::rowCount ( const QModelIndex & parent  ) const
{
	Q_UNUSED(parent);
	//			DBG("MlfSortFilterProxyModel::rowCount()");
	//return sourceModel()->rowCount(parent); 
	//DBG("MlfSortFilterProxyModel::rowCount() = "<<mSortingThread.GetMapToSourceSize());
	return mSortingThread.GetMapToSourceSize();
}
QModelIndex MlfSortFilterProxyModel::parent ( const QModelIndex & index ) const
{
	//			DBG("MlfSortFilterProxyModel::parent()");
	return sourceModel()->parent(index);
}

QVariant MlfSortFilterProxyModel::data ( const QModelIndex & index, int role ) const
{
    if (!index.isValid())
        return QVariant();
	//DBG("MlfSortFilterProxyModel::data["<<index.row()<<","<<index.column()<<"]" );
	return sourceModel()->data(mapToSource(index), role);
}

QVariant MlfSortFilterProxyModel::headerData(int section, Qt::Orientation orientation, int role) const
{
	//DBG("MlfSortFilterProxyModel::headerData("<<section<<", "<<orientation<<")");
	if (orientation == Qt::Horizontal)
	{
		assert(mColumns[section] >= 0);
//		return (QVariant)mColumns.GetColumnLabel((MlfModelData::EColumn)mColumns[section]);
		return sourceModel()->headerData(mColumns[section], orientation, role);
	}
	else
	{
		return sourceModel()->headerData(section, orientation, role);
	}
}

QModelIndex MlfSortFilterProxyModel::index ( int row, int column, const QModelIndex & parent ) const
{
	Q_UNUSED(parent);
	//DBG("MlfSortFilterProxyModel::index("<<row<<", "<<column<<")");

	if ((unsigned int)row<mSortingThread.GetMapSize())
	{
//		return sourceModel()->index(mSortingThread.MapToSource(row), column, parent);
		return createIndex(row, column);
	}
	else
	{
		return QModelIndex();
	}
}


MlfModelData::MlfTableRow * MlfSortFilterProxyModel::GetRow(int row)
{
	return (*mpMlfModel->GetData())[mSortingThread.MapToSource(row)];
}

void MlfSortFilterProxyModel::SortProgressStart()
{
	mpProgressDisplay->SortProgressStart();
}

void MlfSortFilterProxyModel::SortProgress(int percent)
{
	DBG("MlfSortFilterProxyModel::SortProgress("<<percent<<")");
	mpProgressDisplay->SortProgress(percent);
}

void MlfSortFilterProxyModel::setSourceModel ( MlfModel * sourceModel )
{
	QAbstractProxyModel::setSourceModel(sourceModel);
	mpMlfModel = sourceModel;
//	connect(sourceModel, SIGNAL(rowsInserted(const QModelIndex&, int, int)), this, SLOT(SynchronizeMap()));
//	connect(sourceModel, SIGNAL(rowsRemoved(const QModelIndex&, int, int)), this, SLOT(SynchronizeMap()));
//	connect(sourceModel, SIGNAL(dataChanged(const QModelIndex&, const QModelIndex&)), this, SLOT(SynchronizeMap()));
	mSortingThread.SetData(mpMlfModel->GetData());
	SynchronizeMap();
	mpMlfModel->SetSelectedKeywords(mSortingThread.GetSelectedKeywordsMap());
	connect(&mSortingThread, SIGNAL(Sorted()), this, SLOT(Sorted()));
	connect(mpMlfModel, SIGNAL(dataChanged(const QModelIndex&, const QModelIndex&)), this, SLOT(OnDataChanged(const QModelIndex&, const QModelIndex&)));
}

QModelIndex MlfSortFilterProxyModel::mapFromSource ( const QModelIndex & sourceIndex ) const
{
	return sourceModel()->index(mSortingThread.MapFromSource(sourceIndex.row()), sourceIndex.column());
}

QModelIndex MlfSortFilterProxyModel::mapToSource ( const QModelIndex & proxyIndex ) const
{
//	DBG("mapToSource["<<proxyIndex.row()<<", "<<proxyIndex.column()<<"] -> ["<<mSortingThread.MapToSource(proxyIndex.row())<<", "<<mColumns[proxyIndex.column()]<<"] -> ["<<sourceModel()->index(mSortingThread.MapToSource(proxyIndex.row()), mColumns[proxyIndex.column()]).row()<<", "<<sourceModel()->index(mSortingThread.MapToSource(proxyIndex.row()), mColumns[proxyIndex.column()]).column()<<"]");
	assert(mColumns[proxyIndex.column()] >= 0);
	return sourceModel()->index(mSortingThread.MapToSource(proxyIndex.row()), mColumns[proxyIndex.column()]);
}

QItemSelection MlfSortFilterProxyModel::mapSelectionToSource(const QItemSelection &proxySelection) const
{
    return QAbstractProxyModel::mapSelectionToSource(proxySelection);
}

QItemSelection MlfSortFilterProxyModel::mapSelectionFromSource(const QItemSelection &sourceSelection) const
{
    return QAbstractProxyModel::mapSelectionFromSource(sourceSelection);
}

void MlfSortFilterProxyModel::Resort(int column)
{
	if (mColumn == column)
	{
		mpProgressDisplay->SortProgressStart();
		mSortingThread.Sort(mColumn, mOrder, true);
	}
}

void MlfSortFilterProxyModel::sort ( int column, Qt::SortOrder order )
{
	Q_UNUSED(order);
	DBG("MlfSortFilterProxyModel::sort(column="<<column<<")" );

	mpProgressDisplay->SortProgressStart();
	mColumn = column;
	mOrder = order;
	mSortingThread.Sort(column, order);

}

bool MlfSortFilterProxyModel::lessThan(const QModelIndex &left,
		const QModelIndex &right) const
{
	QVariant leftData = sourceModel()->data(left);
	QVariant rightData = sourceModel()->data(right);

	//DBG("col:"<<left.column()<<" "<< leftData.type() );
	if (leftData.type() == QVariant::Double || left.column() == MlfModelData::EColScoreRaw || left.column() == MlfModelData::EColScorePosterior || left.column() == MlfModelData::EColScoreLlr || left.column() == MlfModelData::EColScoreLlrNorm || left.column() == MlfModelData::EColScoreNorm) {
		return leftData.toDouble() < rightData.toDouble();
	} else {
		QString leftString = leftData.toString();
		QString rightString = rightData.toString();
		return QString::localeAwareCompare(leftString, rightString) < 0;
	}
}

void MlfSortFilterProxyModel::SynchronizeMap()
{
	mSortingThread.SynchronizeMap();
}

void MlfSortFilterProxyModel::Sorted()
{
	DBG("MlfSortFilterProxyModel::Sorted()");
	mpProgressDisplay->SortProgressEnd();
	emit dataChanged(index(0,0), index(rowCount()-1, columnCount()));
}


void MlfSortFilterProxyModel::OnDataChanged(const QModelIndex& topLeft, const QModelIndex& bottomRight)
{
	DBG("MlfSortFilterProxyModel::OnDataChanged(["<<topLeft.row()<<","<<topLeft.column()<<"] ["<<bottomRight.row()<<","<<bottomRight.column()<<"])");
	QModelIndex bottom_right = bottomRight;
	if (bottom_right.row() > rowCount())
	{
		bottom_right = createIndex(rowCount(), bottom_right.column());
	}
	if (topLeft.column() <= mColumn && mColumn <= bottom_right.column() && mColumn != MlfModelData::EColScoreRaw && mColumn != MlfModelData::EColScorePosterior && mColumn != MlfModelData::EColScoreLlr && mColumn != MlfModelData::EColScoreLlrNorm && mColumn != MlfModelData::EColScoreNorm)
	{
		mSortingThread.SynchronizeMap();
		sort(mColumn, mOrder);
	}
	else if (rowCount() > 0)
	{
		DBG("emit dataChanged()");
		emit dataChanged(createIndex(0,0), createIndex(rowCount()-1,bottomRight.column()));
	}
}

void MlfSortFilterProxyModel::FilterChanged()
{
	DBG("MlfSortFilterProxyModel::FilterChanged()");

	int old_count = rowCount();
	mSortingThread.FilterChanged();
	mpMlfModel->RefreshHistogram();

	DBG("old_count:"<<old_count<<" new_count:"<<rowCount());

	if (old_count > rowCount())
	{
		emit beginRemoveRows(QModelIndex(), 0, old_count-rowCount()-1);
		emit endRemoveRows();
	}
	else if (old_count < rowCount())
	{
		emit beginInsertRows(QModelIndex(), 0, rowCount()-old_count-1);
		emit endInsertRows();
	}
}

void MlfSortFilterProxyModel::ColumnsChanged()
{
	DBG("MlfSortFilterProxyModel::ColumnsChanged() mColumnsCountPrev:"<<mColumnsCountPrev<<" current:"<<mColumns.Size());
	if (mColumnsCountPrev > mColumns.Size())
	{
		int count = mColumnsCountPrev - mColumns.Size();
		DBG("removeColumns(): "<<count);
		beginRemoveColumns(QModelIndex(), 1, count);
		endRemoveColumns();
	}
	else if (mColumnsCountPrev < mColumns.Size())
	{
		int count = mColumns.Size() - mColumnsCountPrev;
		DBG("insertColumns(): "<<mColumns.Size() - mColumnsCountPrev);
		beginInsertColumns(QModelIndex(), 1, count);
		endInsertColumns();
	}
	mColumnsCountPrev = mColumns.Size();
}

void MlfSortFilterProxyModel::ChangeCorrect(int row)
{
	mpMlfModel->ChangeCorrect(mSortingThread.MapToSource(row));
}

