#ifndef MLFSORTFILTERPROXYMODEL_H
#define MLFSORTFILTERPROXYMODEL_H

#include <QAbstractProxyModel>
#include <QItemSelection>
#include <QVector>
#include <QSettings>
#include <iostream>
#include "mlfmodel.h"
#include "mlfsortingthread.h"
#include "dbg.h"	

class MlfSortFilterProxyModel : public QAbstractProxyModel
{
		Q_OBJECT
	public:
		MlfSortFilterProxyModel (QSettings * pSettings = NULL, QObject * parent = 0);

		void SetProgressDisplay(ISortProgressDisplay * pProgressDisplay) 
		{ 
			mpProgressDisplay = pProgressDisplay;
		}

		QModelIndex mapFromSource ( const QModelIndex & sourceIndex ) const;
		QModelIndex mapToSource ( const QModelIndex & proxyIndex ) const;

		QItemSelection mapSelectionToSource(const QItemSelection &proxySelection) const;
		QItemSelection mapSelectionFromSource(const QItemSelection &sourceSelection) const;

		void setSourceModel ( MlfModel * sourceModel );

		int columnCount ( const QModelIndex & parent = QModelIndex() ) const;
		int rowCount ( const QModelIndex & parent = QModelIndex() ) const;
		QVariant data ( const QModelIndex & index, int role = Qt::DisplayRole ) const;
		QModelIndex index ( int row, int column, const QModelIndex & parent = QModelIndex() ) const;
		QModelIndex parent ( const QModelIndex & index ) const;
		QVariant headerData(int section, Qt::Orientation orientation, int role) const;

		MlfModelData::MlfTableRow * GetRow(int row);
		MlfModelData::Columns * GetColumns() { return &mColumns; }

		void Resort(int column);
		void sort ( int column, Qt::SortOrder order = Qt::AscendingOrder );
		bool lessThan(const QModelIndex &left, const QModelIndex &right) const;

		MlfSortingThread::SelectedKeywordsMap* GetSelectedKeywordsMap() { return mSortingThread.GetSelectedKeywordsMap(); }

		void SetStableSort(bool stable) { mSortingThread.SetStableSort(stable); }
		void SetShowOnlyHD1(bool hd1only) { mSortingThread.SetShowOnlyHD1(hd1only); }

//		void SetFilterLabelsList(QVector<QString>)
	

	public slots:
		void SynchronizeMap();
		void SortProgressStart();
		void SortProgress(int percent);
		void Sorted();
		void OnDataChanged(const QModelIndex& topLeft, const QModelIndex& bottomRight);
		void FilterChanged();
		void ChangeCorrect(int row);
		void ColumnsChanged();

	private:
		MlfModel* mpMlfModel;
		MlfSortingThread mSortingThread;
		QSettings * mpSettings;
		int mColumn;
		MlfModelData::Columns mColumns;
		Qt::SortOrder mOrder;
//		QVector<unsigned int> mMapToSource;
//		QVector<unsigned int> mMapFromSource;
		ISortProgressDisplay * mpProgressDisplay;
		unsigned int mColumnsCountPrev;
};

#endif
