#ifndef MLFSORTINGTHREAD_H
#define MLFSORTINGTHREAD_H

#include <QMutex>
#include <QSize>
#include <QThread>
#include <QWaitCondition>
#include "mlfmodeldata.h"
#include "dbg.h"

class ISortProgressDisplay
{
	public:
		virtual void SortProgressStart() {};
		virtual void SortProgressEnd() {};
		virtual void SortProgress(int percent) = 0;
};

class MlfSortingThread : public QThread
{
		Q_OBJECT
	public:
		typedef QMap<QString, bool> SelectedKeywordsMap;

	    MlfSortingThread(QObject *parent = 0);
	    ~MlfSortingThread();

		void Sort(int column, Qt::SortOrder order, bool force = false);
		void SetData(MlfModelData::Data* pData);
		void SetColumns(MlfModelData::Columns* pColumns);
		void SynchronizeMap();
		unsigned int GetMapSize() const { return mpMapToSource->size(); }

		void SetStableSort(bool stable) { mStableSort = stable; }
		void SetShowOnlyHD1(bool hd1only) { mShowOnlyHD1 = hd1only; }

		SelectedKeywordsMap* GetSelectedKeywordsMap() { return &mSelectedKeywordsMap; }
		unsigned int GetMapToSourceSize() const { return mpMapToSource->size(); }

		unsigned int MapToSource(unsigned int i) const;
		unsigned int MapFromSource(unsigned int i) const;

		static bool CmpFilenameAsc(const MlfModelData::MlfTableRow &a, const MlfModelData::MlfTableRow &b);
		static bool CmpFilenameAscStable(const MlfModelData::MlfTableRow &a, const MlfModelData::MlfTableRow &b);
		static bool CmpFilenameDesc(const MlfModelData::MlfTableRow &a, const MlfModelData::MlfTableRow &b);
		static bool CmpFilenameDescStable(const MlfModelData::MlfTableRow &a, const MlfModelData::MlfTableRow &b);
		static bool CmpStartTimeAsc(const MlfModelData::MlfTableRow &a, const MlfModelData::MlfTableRow &b);
		static bool CmpStartTimeAscStable(const MlfModelData::MlfTableRow &a, const MlfModelData::MlfTableRow &b);
		static bool CmpStartTimeDesc(const MlfModelData::MlfTableRow &a, const MlfModelData::MlfTableRow &b);
		static bool CmpStartTimeDescStable(const MlfModelData::MlfTableRow &a, const MlfModelData::MlfTableRow &b);
		static bool CmpEndTimeAsc(const MlfModelData::MlfTableRow &a, const MlfModelData::MlfTableRow &b);
		static bool CmpEndTimeAscStable(const MlfModelData::MlfTableRow &a, const MlfModelData::MlfTableRow &b);
		static bool CmpEndTimeDesc(const MlfModelData::MlfTableRow &a, const MlfModelData::MlfTableRow &b);
		static bool CmpEndTimeDescStable(const MlfModelData::MlfTableRow &a, const MlfModelData::MlfTableRow &b);
		static bool CmpLabelAsc(const MlfModelData::MlfTableRow &a, const MlfModelData::MlfTableRow &b);
		static bool CmpLabelAscStable(const MlfModelData::MlfTableRow &a, const MlfModelData::MlfTableRow &b);
		static bool CmpLabelDesc(const MlfModelData::MlfTableRow &a, const MlfModelData::MlfTableRow &b);
		static bool CmpLabelDescStable(const MlfModelData::MlfTableRow &a, const MlfModelData::MlfTableRow &b);
		static bool CmpScoreRawAsc(const MlfModelData::MlfTableRow &a, const MlfModelData::MlfTableRow &b);
		static bool CmpScoreRawAscStable(const MlfModelData::MlfTableRow &a, const MlfModelData::MlfTableRow &b);
		static bool CmpScoreRawDesc(const MlfModelData::MlfTableRow &a, const MlfModelData::MlfTableRow &b);
		static bool CmpScoreRawDescStable(const MlfModelData::MlfTableRow &a, const MlfModelData::MlfTableRow &b);
		static bool CmpScorePosteriorAsc(const MlfModelData::MlfTableRow &a, const MlfModelData::MlfTableRow &b);
		static bool CmpScorePosteriorAscStable(const MlfModelData::MlfTableRow &a, const MlfModelData::MlfTableRow &b);
		static bool CmpScorePosteriorDesc(const MlfModelData::MlfTableRow &a, const MlfModelData::MlfTableRow &b);
		static bool CmpScorePosteriorDescStable(const MlfModelData::MlfTableRow &a, const MlfModelData::MlfTableRow &b);
		static bool CmpScoreLlrAsc(const MlfModelData::MlfTableRow &a, const MlfModelData::MlfTableRow &b);
		static bool CmpScoreLlrAscStable(const MlfModelData::MlfTableRow &a, const MlfModelData::MlfTableRow &b);
		static bool CmpScoreLlrDesc(const MlfModelData::MlfTableRow &a, const MlfModelData::MlfTableRow &b);
		static bool CmpScoreLlrDescStable(const MlfModelData::MlfTableRow &a, const MlfModelData::MlfTableRow &b);
		static bool CmpScoreLlrNormAsc(const MlfModelData::MlfTableRow &a, const MlfModelData::MlfTableRow &b);
		static bool CmpScoreLlrNormAscStable(const MlfModelData::MlfTableRow &a, const MlfModelData::MlfTableRow &b);
		static bool CmpScoreLlrNormDesc(const MlfModelData::MlfTableRow &a, const MlfModelData::MlfTableRow &b);
		static bool CmpScoreLlrNormDescStable(const MlfModelData::MlfTableRow &a, const MlfModelData::MlfTableRow &b);
		static bool CmpScoreNormalizedAsc(const MlfModelData::MlfTableRow &a, const MlfModelData::MlfTableRow &b);
		static bool CmpScoreNormalizedAscStable(const MlfModelData::MlfTableRow &a, const MlfModelData::MlfTableRow &b);
		static bool CmpScoreNormalizedDesc(const MlfModelData::MlfTableRow &a, const MlfModelData::MlfTableRow &b);
		static bool CmpScoreNormalizedDescStable(const MlfModelData::MlfTableRow &a, const MlfModelData::MlfTableRow &b);
		static bool CmpCorrectAsc(const MlfModelData::MlfTableRow &a, const MlfModelData::MlfTableRow &b);
		static bool CmpCorrectAscStable(const MlfModelData::MlfTableRow &a, const MlfModelData::MlfTableRow &b);
		static bool CmpCorrectDesc(const MlfModelData::MlfTableRow &a, const MlfModelData::MlfTableRow &b);
		static bool CmpCorrectDescStable(const MlfModelData::MlfTableRow &a, const MlfModelData::MlfTableRow &b);

		typedef QMap<unsigned int, unsigned int> SortingMap;
	signals:
		void SortProgressStart();
		void SortProgress(int percent);
		void Sorted();

	public slots:
		void FilterChanged();

	protected:
		void run();

		QMutex mMutex;
		QWaitCondition mCondition;

		bool mRestart;
		bool mAbort;
		bool mRunning;

		int mColumn;
		int mColumnPrev;
		bool mForce;
		Qt::SortOrder mOrder;
		int mOrderPrev;
		bool mInvert;
		unsigned int mProgressCur;
		unsigned int mProgressMax;
		unsigned int mProgressPercent;
		bool mStableSort;
		bool mShowOnlyHD1;
		bool mFilterChanged;
		unsigned int mSelectedRowsCount;

		MlfModelData::Data * mpData;

		MlfModelData::Columns * mpColumns;

		SortingMap* mpMapToSource;
		SortingMap* mpMapFromSource;

		QMap<QString, bool> mSelectedKeywordsMap;

		void Swap(SortingMap& mapToSource, SortingMap& mapFromSource, int i, int j);

		void BubbleSort
		(
			const MlfModelData::Data& modelData,
			SortingMap& mapToSource,
			SortingMap& mapFromSource,
			bool (*fCmpLess)(const MlfModelData::MlfTableRow &a, const MlfModelData::MlfTableRow &b)
		);

		void QuickSort
		(	
			const MlfModelData::Data& modelData,
			SortingMap& mapToSource,
			SortingMap& mapFromSource,
			int first, 
			int last,
			bool (*fCmpLess)(const MlfModelData::MlfTableRow &a, const MlfModelData::MlfTableRow &b)
		);

		void MergeSort
		(
			 const MlfModelData::Data& modelData,
			 SortingMap& mapToSource,
			 SortingMap& mapFromSource,
			 int first, 
			 int last,
			 bool (*fCmpLess)(const MlfModelData::MlfTableRow &a, const MlfModelData::MlfTableRow &b)
		);
};

#endif

