#include "opendialog.h"
#include <QtGui>

OpenDialog::OpenDialog(QWidget * parent, Qt::WindowFlags f) : QDialog(parent, f)
{
	mpEditMlf = new QLineEdit;
	mpEditRefMlf = new QLineEdit;
	mpEditWavs = new QLineEdit;
	QPushButton * buttonOpenMlf = new QPushButton(tr("Open MLF"));
	QPushButton * buttonOpenRefMlf = new QPushButton(tr("Open reference MLF"));
	QPushButton * buttonOpenWavs = new QPushButton(tr("Set audio folder"));
	QPushButton * buttonOk = new QPushButton(style()->standardIcon(QStyle::SP_DialogOkButton), tr("Ok"));
	QPushButton * buttonCancel = new QPushButton(style()->standardIcon(QStyle::SP_DialogCancelButton), tr("Cancel"));

	connect(buttonOpenMlf, SIGNAL(clicked()), this, SLOT(OpenMlf()));
	connect(buttonOpenRefMlf, SIGNAL(clicked()), this, SLOT(OpenRefMlf()));
	connect(buttonOpenWavs, SIGNAL(clicked()), this, SLOT(OpenWavs()));
	connect(buttonOk, SIGNAL(clicked()), this, SLOT(OkClicked()));
	connect(buttonCancel, SIGNAL(clicked()), this, SLOT(CancelClicked()));

	QGridLayout * layoutPaths = new QGridLayout();

	layoutPaths->addWidget(mpEditMlf,0,0);
	layoutPaths->addWidget(buttonOpenMlf,0,1);

	layoutPaths->addWidget(mpEditRefMlf,1,0);
	layoutPaths->addWidget(buttonOpenRefMlf,1,1);

	layoutPaths->addWidget(mpEditWavs,2,0);
	layoutPaths->addWidget(buttonOpenWavs,2,1);

	QHBoxLayout * layoutOkCancel = new QHBoxLayout;
	layoutOkCancel->addStretch();
	layoutOkCancel->addWidget(buttonOk);
	layoutOkCancel->addWidget(buttonCancel);
	layoutOkCancel->addStretch();

	QVBoxLayout * layoutMain = new QVBoxLayout;
	layoutMain->addLayout(layoutPaths);
	layoutMain->addSpacing(20);
	layoutMain->addLayout(layoutOkCancel);

	setLayout(layoutMain);

	QRect screen = QApplication::desktop()->screenGeometry();
	int width = screen.width()*0.6;
	int height = 150;
	int x = (screen.width()-width) / 2;
	int y = (screen.height()-height) / 2;
	setGeometry(x, y, width, height);
//	setWindowModality(Qt::ApplicationModal);
	setModal(true);
	setFocusPolicy(Qt::StrongFocus);
}

void OpenDialog::exec()
{
	mPrevMlf = mpEditMlf->text();
	mPrevRefMlf = mpEditRefMlf->text();
	mPrevWavs = mpEditWavs->text();
	(static_cast<QDialog*>(this))->exec();
}

void OpenDialog::OkClicked()
{
	DBG("GetMlfFilename():"<<GetMlfFilename().toStdString());
	close();
}

void OpenDialog::CancelClicked()
{
	mpEditMlf->setText(mPrevMlf);
	mpEditRefMlf->setText(mPrevRefMlf);
	mpEditWavs->setText(mPrevWavs);
	close();
}

QString OpenDialog::GetMlfFilename() const
{
	return mpEditMlf->text();
}

QString OpenDialog::GetRefMlfFilename() const
{
	return mpEditRefMlf->text();
}

QString OpenDialog::GetWavsPath() const
{
	return mpEditWavs->text();
}

void OpenDialog::OpenMlf()
{
	QString filename = QFileDialog::getOpenFileName(this, tr("Open MLF"), "", tr("MLF Files (*.mlf)"));
	if (filename != "")
	{
		mpEditMlf->setText(filename);
	}
}

void OpenDialog::OpenRefMlf()
{
	QString filename = QFileDialog::getOpenFileName(this, tr("Open reference MLF"), "", tr("MLF Files (*.mlf)"));
	if (filename != "")
	{
		mpEditRefMlf->setText(filename);
	}
}

void OpenDialog::OpenWavs()
{
	QString dir = QFileDialog::getExistingDirectory(this, tr("Select WAV files directory"), "", QFileDialog::ShowDirsOnly | QFileDialog::DontResolveSymlinks);
	DBG("AudioFilesDirectory:"<<dir.toStdString());
	if (dir != "")
	{
		mpEditWavs->setText(dir);
	}
}

void OpenDialog::keyPressEvent(QKeyEvent * event)
{
	switch(event->key())
	{
		case Qt::Key_Escape: CancelClicked(); break;
	}
}

