#include "settingsdialog.h"
#include <QtGui>

SettingsDialog::SettingsDialog(QWidget * parent, Qt::WindowFlags f) : QDialog(parent, f)
{
	mContextValue = 0;
	mStableSortCheckedPrev = false;

	mpRadiobuttonEncodingISO88592 = new QRadioButton(tr("ISO 8859-2"));
	mpRadiobuttonEncodingCP1250 = new QRadioButton(tr("CP 1250"));
	mpRadiobuttonEncodingCP1251 = new QRadioButton(tr("CP 1251"));
	mpRadiobuttonEncodingUTF8 = new QRadioButton(tr("UTF-8"));
	mpRadiobuttonEncodingCP1250->setChecked(true);

	mpRadiobuttonInputscoretypeLogprob = new QRadioButton(tr("Log-probability"));
	mpRadiobuttonInputscoretypeLlr = new QRadioButton(tr("Log-likelihood ratio"));
	mpRadiobuttonInputscoretypeLogprob->setChecked(true);

	mpEditAudioFileSuffix = new QLineEdit;
	mpCheckboxIsRawAudio = new QCheckBox(tr(""));
	mpComboboxFrequency = new QComboBox;
	mpComboboxFrequency->addItem("8000 Hz", (int)8000);
	mpComboboxFrequency->addItem("16000 Hz", (int)16000);
	mpComboboxFrequency->addItem("32000 Hz", (int)32000);
	mpComboboxFrequency->addItem("44100 Hz", (int)44100);
	mpComboboxFrequency->addItem("48000 Hz", (int)48000);
	mpSpinboxNumChannels = new QSpinBox;
	mpComboboxRawFormat = new QComboBox;
	mpComboboxRawFormat->addItem("8bit integer PCM data", (int)FMOD_SOUND_FORMAT_PCM8);
	mpComboboxRawFormat->addItem("16bit integer PCM data", (int)FMOD_SOUND_FORMAT_PCM16);
	mpComboboxRawFormat->addItem("24bit integer PCM data", (int)FMOD_SOUND_FORMAT_PCM24);
	mpComboboxRawFormat->addItem("32bit integer PCM data", (int)FMOD_SOUND_FORMAT_PCM32);
	mpComboboxRawFormat->addItem("32bit floating point PCM data", (int)FMOD_SOUND_FORMAT_PCMFLOAT);

	mpCheckboxStableSort = new QCheckBox(tr("Multiple column sorting"));
	mpCheckboxStableSort->setToolTip(tr("Slower, but allows the table to be sorted by multiple columns"));

	QLabel * labelRowHeight = new QLabel(tr("Table row height"));
	mpSpinboxRowHeight = new QSpinBox;
	mpSpinboxRowHeight->setRange(1, 40);

//	mpCheckboxHistogramIgiAccumulative = new QCheckBox(tr("Igi accumulative"));
//	mpCheckboxHistogramLukasAccumulative = new QCheckBox(tr("Lukas accumulative"));
//	mpCheckboxHistogramLukas = new QCheckBox(tr("Lukas"));
	mpCheckboxHistogramIgiAccumulative = new QRadioButton(tr("0..1 score - cumulative histogram"));
	mpCheckboxHistogramLukasAccumulative = new QRadioButton(tr("LLR - CDF"));
	mpCheckboxHistogramLukas = new QRadioButton(tr("Lukas"));

	for (int i=0; i<MlfModelData::EColCOUNT; i++)
	{
		mpCheckboxColumns[i] = new QCheckBox(MlfModelData::Columns::GetColumnLabel((MlfModelData::EColumn)i));
	}

	create_slider_spinbox_pair(&mpSliderReferenceBoundary, &mpSpinboxReferenceBoundary);
	set_slider_spinbox_pair(mpSliderReferenceBoundary, mpSpinboxReferenceBoundary, mReferenceBoundaryMin, mReferenceBoundaryMax, mReferenceBoundaryPrecision, mReferenceBoundaryDefaultValue, &mReferenceBoundaryValue);

	create_slider_spinbox_pair(&mpSliderContext, &mpSpinboxContext);
	set_slider_spinbox_pair(mpSliderContext, mpSpinboxContext, mContextMin, mContextMax, mContextPrecision, mContextDefaultValue, &mContextValue);

	QPushButton * buttonOk = new QPushButton(style()->standardIcon(QStyle::SP_DialogOkButton), tr("Ok"));
	QPushButton * buttonCancel = new QPushButton(style()->standardIcon(QStyle::SP_DialogCancelButton), tr("Cancel"));

//	connect(mpRadiobuttonEncodingISO88592, SIGNAL(toggled(bool)), this, SLOT(RadiobuttonEncodingISO88592Toggled(bool)));
//	connect(mpRadiobuttonEncodingCP1250, SIGNAL(toggled(bool)), this, SLOT(RadiobuttonEncodingCP1250Toggled(bool)));
//	connect(mpCheckboxStableSort, SIGNAL(toggled(bool)), this, SLOT(CheckboxStableSortToggled(bool)));
	connect(mpSliderContext, SIGNAL(valueChanged(int)), this, SLOT(ContextSliderChanged(int)));
	connect(mpSpinboxContext, SIGNAL(valueChanged(double)), this, SLOT(ContextSpinboxChanged(double)));
	connect(mpSliderReferenceBoundary, SIGNAL(valueChanged(int)), this, SLOT(ReferenceBoundarySliderChanged(int)));
	connect(mpSpinboxReferenceBoundary, SIGNAL(valueChanged(double)), this, SLOT(ReferenceBoundarySpinboxChanged(double)));
	connect(mpSpinboxRowHeight, SIGNAL(valueChanged(int)), this, SLOT(RowHeightSpinboxChanged(int)));
	connect(mpCheckboxIsRawAudio, SIGNAL(stateChanged(int)), this, SLOT(IsRawAudioCheckboxChanged(int)));
	connect(buttonOk, SIGNAL(clicked()), this, SLOT(OkClicked()));
	connect(buttonCancel, SIGNAL(clicked()), this, SLOT(CancelClicked()));

	QHBoxLayout *layoutEncoding = new QHBoxLayout;
	layoutEncoding->addWidget(mpRadiobuttonEncodingISO88592);
	layoutEncoding->addWidget(mpRadiobuttonEncodingCP1250);
	layoutEncoding->addWidget(mpRadiobuttonEncodingCP1251);
	layoutEncoding->addWidget(mpRadiobuttonEncodingUTF8);
	QGroupBox* groupboxEncoding = new QGroupBox(tr("Encoding"));
	groupboxEncoding->setLayout(layoutEncoding);

	QHBoxLayout *layoutInputscoretype = new QHBoxLayout;
	layoutInputscoretype->addWidget(mpRadiobuttonInputscoretypeLogprob);
	layoutInputscoretype->addWidget(mpRadiobuttonInputscoretypeLlr);
	QGroupBox* groupboxInputscoretype = new QGroupBox(tr("Input score type (in MLF)"));
	groupboxInputscoretype->setLayout(layoutInputscoretype);

	QGridLayout *layoutAudiofiles = new QGridLayout;
	layoutAudiofiles->addWidget(new QLabel(tr("audio files extension (wav)")), 0, 0);
	layoutAudiofiles->addWidget(mpEditAudioFileSuffix, 0, 1);
	layoutAudiofiles->addWidget(new QLabel(tr("raw audio files")), 1, 0);
	layoutAudiofiles->addWidget(mpCheckboxIsRawAudio, 1, 1);
	layoutAudiofiles->addWidget(new QLabel(tr("frequency")), 2, 0);
	layoutAudiofiles->addWidget(mpComboboxFrequency, 2, 1);
	layoutAudiofiles->addWidget(new QLabel(tr("number of channels")), 3, 0);
	layoutAudiofiles->addWidget(mpSpinboxNumChannels, 3, 1);
	layoutAudiofiles->addWidget(new QLabel(tr("raw format")), 4, 0);
	layoutAudiofiles->addWidget(mpComboboxRawFormat, 4, 1);
	QGroupBox* groupboxAudiofiles = new QGroupBox(tr("Audio files settings"));
	groupboxAudiofiles->setLayout(layoutAudiofiles);

	QHBoxLayout *layoutContext = new QHBoxLayout;
	layoutContext->addWidget(mpSliderContext);
	layoutContext->addWidget(mpSpinboxContext);
	QGroupBox* groupboxContext = new QGroupBox(tr("Detection context [seconds] for audio player"));
	groupboxContext->setLayout(layoutContext);

	QHBoxLayout *layoutReferenceBoundary = new QHBoxLayout;
	layoutReferenceBoundary->addWidget(mpSliderReferenceBoundary);
	layoutReferenceBoundary->addWidget(mpSpinboxReferenceBoundary);
	QGroupBox* groupboxReferenceBoundary = new QGroupBox(tr("Reference boundary [seconds]"));
	groupboxReferenceBoundary->setLayout(layoutReferenceBoundary);
	groupboxReferenceBoundary->setToolTip(tr("Each item in the reference will be enlarged on both ends by this value when computing Hits and Misses"));

	QHBoxLayout *layoutRowHeight = new QHBoxLayout;
	layoutRowHeight->addWidget(labelRowHeight);
	layoutRowHeight->addWidget(mpSpinboxRowHeight);

	QHBoxLayout *layoutHistogramType = new QHBoxLayout;
	layoutHistogramType->addWidget(mpCheckboxHistogramIgiAccumulative);
	layoutHistogramType->addWidget(mpCheckboxHistogramLukasAccumulative);
//	layoutHistogramType->addWidget(mpCheckboxHistogramLukas);
	QGroupBox *groupboxHistogramType = new QGroupBox(tr("Plot type"));
	groupboxHistogramType->setLayout(layoutHistogramType);

	QVBoxLayout *layoutColumns = new QVBoxLayout;
	for (int i=0; i<MlfModelData::EColCOUNT; i++)
	{
		layoutColumns->addWidget(mpCheckboxColumns[i]);
	}
	QGroupBox* groupboxColumns = new QGroupBox(tr("Columns"));
	groupboxColumns->setLayout(layoutColumns);

	QVBoxLayout * layoutColumnLeft = new QVBoxLayout;
//	layoutColumnLeft->addWidget(mpCheckboxStableSort);
	layoutColumnLeft->addWidget(groupboxEncoding);
	layoutColumnLeft->addWidget(groupboxInputscoretype);
	layoutColumnLeft->addWidget(groupboxAudiofiles);
	layoutColumnLeft->addWidget(groupboxContext);
	layoutColumnLeft->addWidget(groupboxReferenceBoundary);
	layoutColumnLeft->addWidget(groupboxHistogramType);
	layoutColumnLeft->addLayout(layoutRowHeight);
	layoutColumnLeft->addStretch();

	QVBoxLayout * layoutColumnRight = new QVBoxLayout;
	layoutColumnRight->addWidget(groupboxColumns);
	layoutColumnRight->addStretch();

	QHBoxLayout * layoutControls = new QHBoxLayout;
	layoutControls->addLayout(layoutColumnLeft);
	layoutControls->addSpacing(30);
	layoutControls->addLayout(layoutColumnRight);

	QHBoxLayout * layoutOkCancel = new QHBoxLayout;
	layoutOkCancel->addStretch();
	layoutOkCancel->addWidget(buttonOk);
	layoutOkCancel->addWidget(buttonCancel);
	layoutOkCancel->addStretch();

	QVBoxLayout * layoutMain = new QVBoxLayout;
	layoutMain->addLayout(layoutControls);
	layoutMain->addSpacing(20);
	layoutMain->addLayout(layoutOkCancel);

	setLayout(layoutMain);

	QRect screen = QApplication::desktop()->screenGeometry();
	int width = 300;
	int height = 450;
	int x = (screen.width()-width) / 2;
	int y = (screen.height()-height) / 2;
	setGeometry(x, y, width, height);
//	setWindowModality(Qt::ApplicationModal);
	setModal(true);
	setFocusPolicy(Qt::StrongFocus);
}

void SettingsDialog::SetColumns(MlfModelData::Columns * pColumns)
{
	mpColumns = pColumns;
	// reset all columns
	for (int i=0; i<MlfModelData::EColCOUNT; i++)
	{
		mpCheckboxColumns[i]->setChecked(false);
	}
	// check selected columns
	for (unsigned int i=0; i<mpColumns->Size(); i++)
	{
		mpCheckboxColumns[(*mpColumns)[i]]->setChecked(true);
	}
}

int SettingsDialog::exec()
{
	mEncodingPrev = GetEncoding();
	mInputScoreTypePrev = GetInputScoreType();
	mStableSortCheckedPrev = GetStableSortChecked();
	mContextValuePrev = GetContextMilliseconds();
	DBG("mContextValuePrev:"<<mContextValuePrev);
	mReferenceBoundaryValuePrev = GetReferenceBoundaryMilliseconds();
	mRowHeightPrev = mpSpinboxRowHeight->value();
	mHistogramTypePrev = GetHistogramType();

	mAudioFileSuffixPrev = GetAudioFileSuffix();
	DBG("mAudioFileSuffixPrev:"<<mAudioFileSuffixPrev.toStdString());
	mIsRawAudioPrev = GetIsRawAudio();
	mRawFrequencyPrev = GetRawFrequency();
	mRawNumChannelsPrev = GetRawNumChannels();
	mRawFormatPrev = GetRawFormat();

/*
	mColumnsPrev.Clear();
	for (int i=0; i<mpColumns->Size(); i++)
	{
		mColumnsPrev.SetColumn(i, (*mpColumns)[i]);
	}
*/
/*
	mColumnFilenamePrev = GetColumnFilenameChecked();
	mColumnStartTimePrev = GetColumnStartTimeChecked();
	mColumnEndTimePrev = GetColumnEndTimeChecked();
	mColumnLabelPrev = GetColumnLabelChecked();
	mColumnScoreRawPrev = GetColumnScoreRawChecked();
	mColumnScoreLlrPrev = GetColumnScoreLlrChecked();
	mColumnScoreNormPrev = GetColumnScoreNormChecked();
	mColumnHardDecisionPrev = GetColumnHardDecisionChecked();
	mColumnCorrectPrev = GetColumnCorrectChecked();
*/
	setResult(QDialog::Rejected);
	return (static_cast<QDialog*>(this))->exec();
}

void SettingsDialog::OkClicked()
{
	mpColumns->Clear();
	for (int i=0; i<MlfModelData::EColCOUNT; i++)
	{
		if(mpCheckboxColumns[i]->isChecked())
		{
			mpColumns->AddColumn((MlfModelData::EColumn)i);
		}
	}
	if (mRowHeightPrev != mpSpinboxRowHeight->value())
	{
		emit RowHeightChanged(mpSpinboxRowHeight->value());
	}
	accept();
}

void SettingsDialog::CancelClicked()
{
	SetEncoding(mEncodingPrev);
	SetInputScoreType(mInputScoreTypePrev);
	SetStableSortChecked(mStableSortCheckedPrev);
	SetContextMilliseconds(mContextValuePrev);
	SetReferenceBoundaryMilliseconds(mReferenceBoundaryValuePrev);
	SetRowHeight(mRowHeightPrev);
	SetHistogramType(mHistogramTypePrev);

	SetAudioFileSuffix(mAudioFileSuffixPrev);
	SetIsRawAudio(mIsRawAudioPrev);
	SetRawFrequency(mRawFrequencyPrev);
	SetRawNumChannels(mRawNumChannelsPrev);
	SetRawFormat(mRawFormatPrev);

//	SetColumnsChecked(mColumnsPrev);
	reject();
}

void SettingsDialog::SetEncoding(QString charset)
{
	if (charset == "CP 1250")
	{
		mpRadiobuttonEncodingCP1250->setChecked(true);
	}
	else if (charset == "CP 1251")
	{
		mpRadiobuttonEncodingCP1251->setChecked(true);
	}
	else if (charset == "ISO 8859-2")
	{
		mpRadiobuttonEncodingISO88592->setChecked(true);
	}
	else if (charset == "UTF8")
	{
		mpRadiobuttonEncodingISO88592->setChecked(true);
	}
}

QString SettingsDialog::GetEncoding()
{
	if (mpRadiobuttonEncodingISO88592->isChecked())
	{
		return "ISO 8859-2";
	}
	else if (mpRadiobuttonEncodingCP1250->isChecked())
	{
		return "CP 1250";
	}
	else if (mpRadiobuttonEncodingCP1251->isChecked())
	{
		return "CP 1251";
	}
	else if (mpRadiobuttonEncodingUTF8->isChecked())
	{
		return "UTF-8";
	}
	else { return ""; }
}

void SettingsDialog::SetInputScoreType(MlfModelData::EInputScoreType type)
{
	if (type == MlfModelData::InputScoreTypeLogprob)
	{
		mpRadiobuttonInputscoretypeLogprob->setChecked(true);
	}
	else if (type == MlfModelData::InputScoreTypeLlr)
	{
		mpRadiobuttonInputscoretypeLlr->setChecked(true);
	}
}

MlfModelData::EInputScoreType SettingsDialog::GetInputScoreType()
{
	if (mpRadiobuttonInputscoretypeLogprob->isChecked())
	{
		return MlfModelData::InputScoreTypeLogprob;
	}
	else if (mpRadiobuttonInputscoretypeLlr->isChecked())
	{
		return MlfModelData::InputScoreTypeLlr;
	}
	return MlfModelData::InputScoreTypeLogprob;
}

bool SettingsDialog::GetStableSortChecked()
{
	return mpCheckboxStableSort->isChecked();
}

void SettingsDialog::SetStableSortChecked(bool checked)
{
	mpCheckboxStableSort->setChecked(checked);
}

void SettingsDialog::ContextSliderChanged(int v)
{
	int mul = pow((float)10,mContextPrecision);
	double f = (double)v/mul;
	if (cmp_float_precision(mContextValue, f, mContextPrecision) != 0)
	{
		mContextValue = f;
		mpSpinboxContext->setValue(f);
	}
	DBG("mContextValue:"<<mContextValue);
}

void SettingsDialog::ContextSpinboxChanged(double v)
{
	if (cmp_float_precision(mContextValue, v, mContextPrecision) != 0)
	{
		mContextValue = v;
		mpSliderContext->setValue(to_int(v*pow((float)10,mContextPrecision)));
	}
	DBG("mContextValue:"<<mContextValue);
}

unsigned int SettingsDialog::GetContextMilliseconds()
{
	DBG("GetContextMilliseconds:"<<(double)mContextValue*1000);
	return (double)mContextValue * 1000;
}

void SettingsDialog::SetContextMilliseconds(unsigned int contextMs)
{
	DBG("SetContextMilliseconds():"<<contextMs<<" -> "<<(double)contextMs/1000);
	mpSpinboxContext->setValue((double)contextMs/1000); // this will change also mContextValue and mpSliderContext
}

void SettingsDialog::ReferenceBoundarySliderChanged(int v)
{
	int mul = pow((float)10,mReferenceBoundaryPrecision);
	double f = (double)v/mul;
	if (cmp_float_precision(mReferenceBoundaryValue, f, mReferenceBoundaryPrecision) != 0)
	{
		mReferenceBoundaryValue = f;
		mpSpinboxReferenceBoundary->setValue(f);
	}
	DBG("mReferenceBoundaryValue:"<<mReferenceBoundaryValue);
}

void SettingsDialog::ReferenceBoundarySpinboxChanged(double v)
{
	if (cmp_float_precision(mReferenceBoundaryValue, v, mReferenceBoundaryPrecision) != 0)
	{
		mReferenceBoundaryValue = v;
		mpSliderReferenceBoundary->setValue(to_int(v*pow((float)10,mReferenceBoundaryPrecision)));
	}
	DBG("mReferenceBoundaryValue:"<<mReferenceBoundaryValue);
}

unsigned int SettingsDialog::GetReferenceBoundaryMilliseconds()
{
	DBG("GetReferenceBoundaryMilliseconds:"<<(double)mReferenceBoundaryValue*1000);
	return (double)mReferenceBoundaryValue * 1000;
}

void SettingsDialog::SetReferenceBoundaryMilliseconds(unsigned int contextMs)
{
	DBG("SetReferenceBoundaryMilliseconds():"<<contextMs<<" -> "<<(double)contextMs/1000);
	mpSpinboxReferenceBoundary->setValue((double)contextMs/1000); // this will change also mReferenceBoundaryValue and mpSliderReferenceBoundary
}

void SettingsDialog::keyPressEvent(QKeyEvent * event)
{
	DBG("keyPressEvent: "<<event->key());
	switch(event->key())
	{
		case Qt::Key_Escape: CancelClicked(); break;
		case Qt::Key_Return: OkClicked(); break;
	}
}

void SettingsDialog::RowHeightSpinboxChanged(int h)
{
	Q_UNUSED(h);
//	emit RowHeightChanged(h);
}

void SettingsDialog::SetRowHeight(int height)
{
	mpSpinboxRowHeight->setValue(height);
}

void SettingsDialog::SetHistogramType(unsigned char type)
{
	if (type & HistogramPlot::EHistogramTypeIgiAccumulative)
	{
		mpCheckboxHistogramIgiAccumulative->setChecked(true);
	}
	if (type & HistogramPlot::EHistogramTypeLukasAccumulative)
	{
		mpCheckboxHistogramLukasAccumulative->setChecked(true);
	}
	if (type & HistogramPlot::EHistogramTypeLukas)
	{
		mpCheckboxHistogramLukas->setChecked(true);
	}
}

unsigned char SettingsDialog::GetHistogramType()
{
	unsigned char res = 0;
	if (mpCheckboxHistogramIgiAccumulative->isChecked())
	{
		res |= HistogramPlot::EHistogramTypeIgiAccumulative;
	}
	if (mpCheckboxHistogramLukasAccumulative->isChecked())
	{
		res |= HistogramPlot::EHistogramTypeLukasAccumulative;
	}
	if (mpCheckboxHistogramLukas->isChecked())
	{
		res |= HistogramPlot::EHistogramTypeLukas;
	}
	return res;
}

void SettingsDialog::IsRawAudioCheckboxChanged(int state)
{
	Q_UNUSED(state);	
	bool isRaw = mpCheckboxIsRawAudio->isChecked();
	mpComboboxFrequency->setEnabled(isRaw);
	mpSpinboxNumChannels->setEnabled(isRaw);
	mpComboboxRawFormat->setEnabled(isRaw);
}

