#ifndef SETTINGSDIALOG_H
#define SETTINGSDIALOG_H

#include <QDialog>
#include <QLineEdit>
#include <QSpinBox>
#include <QCheckBox>
#include <QComboBox>
#include <map>
#include "common.h"
#include "dbg.h"
#include "mlfmodeldata.h"
#include "histogramplot.h"
#include "fmod.h"

class QRadioButton;
class QCheckBox;
class QDoubleSpinBox;
class QSpinBox;
class QSlider;
class QComboBox;
class QLineEdit;

class SettingsDialog : public QDialog
{
	Q_OBJECT
	public:
						SettingsDialog(QWidget * parent = 0, Qt::WindowFlags f = 0);
		void 			SetEncoding(QString charset);
		QString 		GetEncoding();
		void 			SetInputScoreType(MlfModelData::EInputScoreType type);
		MlfModelData::EInputScoreType GetInputScoreType();
		void 			SetStableSortChecked(bool checked);
		bool 			GetStableSortChecked();
		unsigned int 	GetContextMilliseconds();
		void 			SetContextMilliseconds(unsigned int contextMs);
		unsigned int 	GetReferenceBoundaryMilliseconds();
		void 			SetReferenceBoundaryMilliseconds(unsigned int contextMs);
		void 			SetRowHeight(int height);
		QString 		GetAudioFileSuffix() {return mpEditAudioFileSuffix->text();}
		void			SetAudioFileSuffix(QString suffix) {mpEditAudioFileSuffix->setText(suffix);}
		bool 			GetIsRawAudio() {return mpCheckboxIsRawAudio->isChecked();}
		void			SetIsRawAudio(bool isRaw) {mpCheckboxIsRawAudio->setChecked(isRaw); IsRawAudioCheckboxChanged(isRaw);}
		int 			GetRawFrequency() {return mpComboboxFrequency->itemData(mpComboboxFrequency->currentIndex()).toInt();}
		void			SetRawFrequency(int f) {mpComboboxFrequency->setCurrentIndex(mpComboboxFrequency->findData(f));}
		int 			GetRawNumChannels() {return mpSpinboxNumChannels->value();}
		void			SetRawNumChannels(int n) {mpSpinboxNumChannels->setValue(n);}
		int 			GetRawFormat() {return mpComboboxRawFormat->itemData(mpComboboxRawFormat->currentIndex()).toInt();}
		void			SetRawFormat(int fmt) {mpComboboxRawFormat->setCurrentIndex(mpComboboxRawFormat->findData(fmt));}

		void 			SetColumns(MlfModelData::Columns * pColumns);

		void 			SetHistogramType(unsigned char type);
		unsigned char 	GetHistogramType();

	public slots:
		int exec();
		void OkClicked();
		void CancelClicked();
		void ContextSliderChanged(int v);
		void ContextSpinboxChanged(double v);
		void ReferenceBoundarySliderChanged(int v);
		void ReferenceBoundarySpinboxChanged(double v);
		void RowHeightSpinboxChanged(int h);
		void IsRawAudioCheckboxChanged(int state);

	signals:
		void RowHeightChanged(int height);

	protected:
		void keyPressEvent(QKeyEvent * event);

	private:
		QRadioButton *		mpRadiobuttonEncodingISO88592;
		QRadioButton *		mpRadiobuttonEncodingCP1250;
		QRadioButton *		mpRadiobuttonEncodingCP1251;
		QRadioButton *		mpRadiobuttonEncodingUTF8;
		QRadioButton *		mpRadiobuttonInputscoretypeLogprob;
		QRadioButton *		mpRadiobuttonInputscoretypeLlr;
		QCheckBox *			mpCheckboxStableSort;
		QSlider *			mpSliderContext;
		QDoubleSpinBox *	mpSpinboxContext;
		QSlider *			mpSliderReferenceBoundary;
		QDoubleSpinBox *	mpSpinboxReferenceBoundary;
		QSpinBox *			mpSpinboxRowHeight;
//		QCheckBox* 			mpCheckboxHistogramIgiAccumulative;
//		QCheckBox* 			mpCheckboxHistogramLukasAccumulative;
//		QCheckBox* 			mpCheckboxHistogramLukas;
		QRadioButton* 		mpCheckboxHistogramIgiAccumulative;
		QRadioButton* 		mpCheckboxHistogramLukasAccumulative;
		QRadioButton* 		mpCheckboxHistogramLukas;

		QLineEdit *			mpEditAudioFileSuffix;
		QCheckBox *			mpCheckboxIsRawAudio;
		QComboBox *			mpComboboxFrequency;
		QSpinBox *			mpSpinboxNumChannels;
		QComboBox *			mpComboboxRawFormat;

		QCheckBox *			mpCheckboxColumns[MlfModelData::EColCOUNT];

		QString 			mEncodingPrev;
		MlfModelData::EInputScoreType mInputScoreTypePrev;
		bool				mStableSortCheckedPrev;
		unsigned char		mHistogramTypePrev;
		QString				mAudioFileSuffixPrev;
		bool				mIsRawAudioPrev;
		int					mRawFrequencyPrev;
		int					mRawNumChannelsPrev;
		int					mRawFormatPrev;

		int 				mRowHeightPrev;

		MlfModelData::Columns * mpColumns;

//		MlfModelData::Columns mColumnsPrev; // EColumn -> index
/*
		bool				mColumnFilenamePrev;
		bool				mColumnStartTimePrev;
		bool				mColumnEndTimePrev;
		bool				mColumnLabelPrev;
		bool				mColumnScoreRawPrev;
		bool				mColumnScoreLlrPrev;
		bool				mColumnScoreNormPrev;
		bool				mColumnHardDecisionPrev;
		bool				mColumnCorrectPrev;
*/
		double 				mReferenceBoundaryValue;
		double				mReferenceBoundaryValuePrev;

		static const int mReferenceBoundaryMin = 0;
		static const int mReferenceBoundaryMax = 5;
		static const int mReferenceBoundaryPrecision = 1;
		static const int mReferenceBoundaryDefaultValue = 0;

		double 				mContextValue;
		double				mContextValuePrev;

		static const int mContextMin = 0;
		static const int mContextMax = 5;
		static const int mContextPrecision = 1;
		static const int mContextDefaultValue = 0;
};

#endif

