#include <QtGui>
#include <iostream>

#include "tableitemdelegate.h"
#include "mlfmodel.h"

using namespace std;

TableItemDelegate::TableItemDelegate(QObject *parent)
: QItemDelegate(parent), mpMlfModel(NULL), mpProxyModel(NULL), mThreshold(0)
{
	mpIconYes = new QIcon(":/icons/yes.png");
	mpIconNo = new QIcon(":/icons/no.png");
	mpIconUndefined = new QIcon(":/icons/undefined.png");
	mFixedScoreInterval = false;
}

void TableItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option,
		const QModelIndex &index) const
{
	if (mpProxyModel && mpMlfModel)
	{
		//if (index.column() == 6 && index.data().toInt() == 0)
		//DBG("Row:"<<mpProxyModel->mapToSource(index).row());
		MlfModelData::MlfTableRow *r = mpMlfModel->Row(mpProxyModel->mapToSource(index).row());
		if (r)
		{
			double diff = mpMlfModel->GetThreshold() - r->mpMlfRecord->mScore2;
			if (diff < 0) { diff *= -1; }

			double below_values = mpMlfModel->GetThreshold() - (mFixedScoreInterval ? mScoreMin : mpMlfModel->GetNormalizedScoreMin());
			//DBG("below_values = "<<mpMlfModel->GetThreshold()<<" - "<<mpMlfModel->GetNormalizedScoreMin()<<" = "<< below_values );
			double above_values = (mFixedScoreInterval ? mScoreMax : mpMlfModel->GetNormalizedScoreMax()) - mpMlfModel->GetThreshold();
			//DBG("above_values = "<<mpMlfModel->GetNormalizedScoreMax()<<" - "<<mpMlfModel->GetThreshold()<<" = "<< above_values );

			QBrush bg_brush;
			bg_brush.setStyle(Qt::SolidPattern);
			if (option.state & QStyle::State_Selected)
			{
				bg_brush = option.palette.highlight();
			}
			else if (mpMlfModel->GetThreshold() > r->mpMlfRecord->mScore2)
			{
				//DBG("below: 255 / "<<below_values<<" * "<<diff<<" = "<<255/below_values*diff);
				unsigned int i = 55+(int)(200/below_values*diff);
				if (i > 255) { i = 55; }
				bg_brush.setColor(QColor::fromHsv(0, i, 255));
			}
			else
			{
				//DBG("above: 255 / "<<above_values<<" * "<<diff<<" = "<<255/above_values*diff);
				unsigned int i = 55+(int)(200/above_values*diff);
				if (i > 255) { i = 55; }
				bg_brush.setColor(QColor::fromHsv(120, i, 255));
			}

			painter->fillRect(option.rect, bg_brush);
		}
		
		if ((*mpColumns)[index.column()] == MlfModelData::EColCorrect)
		{
			QIcon* icon = NULL;
			switch (index.data().toInt())
			{
				case MlfModelData::CorrectYes: icon = mpIconYes; break;
				case MlfModelData::CorrectNo: icon = mpIconNo; break;
				case MlfModelData::CorrectUndefined: icon = mpIconUndefined; break;
			}
			if (icon)
			{
				painter->save();
				icon->paint(painter, option.rect);
				painter->restore();
			}
//			QItemDelegate::paint(painter, option, index); 
		}
	}
	
	if ((*mpColumns)[index.column()] != MlfModelData::EColCorrect)
	{
		QItemDelegate::paint(painter, option, index); 
	}
/*	
	if (option.state & QStyle::State_Selected)
	{
		painter->save();
		QPen penBorder = painter->pen();
		penBorder.setColor(Qt::darkBlue);
		penBorder.setWidth(5);

//		painter->setBrush(brushSelected);
		painter->setPen(penBorder);

		QRect r = option.rect;
		painter->drawLine(r.left(), r.top(), r.right(), r.top());
		painter->drawLine(r.left(), r.bottom(), r.bottom(), r.bottom());
		painter->restore();
//		option.state = QStyle::State_None;
//		QBrush brushSelected;
//		brushSelected.setStyle(Qt::SolidPattern);
//		brushSelected.setColor(Qt::green);
//		painter->setBrush(brushSelected);
//		DBG("" << option.palette.highlightedText());
	}
*/
}

